from typing import Any, Optional, Union, Text
from .pubkey import pubkey

class _RSAobj(pubkey):
    keydata = ...  # type: Any
    implementation = ...  # type: Any
    key = ...  # type: Any
    def __init__(self, implementation, key, randfunc: Optional[Any] = ...) -> None: ...
    def __getattr__(self, attrname): ...
    def encrypt(self, plaintext, K): ...
    def decrypt(self, ciphertext): ...
    def sign(self, M, K): ...
    def verify(self, M, signature): ...
    def has_private(self): ...
    def size(self): ...
    def can_blind(self): ...
    def can_encrypt(self): ...
    def can_sign(self): ...
    def publickey(self): ...
    def exportKey(self, format: str = ..., passphrase: Optional[Any] = ..., pkcs: int = ...): ...

class RSAImplementation:
    error = ...  # type: Any
    def __init__(self, **kwargs) -> None: ...
    def generate(self, bits, randfunc: Optional[Any] = ..., progress_func: Optional[Any] = ..., e: int = ...): ...
    def construct(self, tup): ...
    def importKey(self, externKey: Any, passphrase: Union[None, bytes, Text] = ...) -> _RSAobj: ...

generate = ...  # type: Any
construct = ...  # type: Any
importKey = ...  # type: Any
error = ...  # type: Any



from typing import Sequence

from mypy.types import (
    Type, UnboundType, AnyType, NoneTyp, TupleType, TypedDictType,
    UnionType, CallableType, TypeVarType, Instance, TypeVisitor, ErasedType,
    Overloaded, PartialType, DeletedType, UninhabitedType, TypeType
)


def is_same_type(left: Type, right: Type) -> bool:
    """Is 'left' the same type as 'right'?"""

    if isinstance(right, UnboundType):
        # Make unbound types same as anything else to reduce the number of
        # generated spurious error messages.
        return True
    else:
        # Simplify types to canonical forms.
        #
        # There are multiple possible union types that represent the same type,
        # such as Union[int, bool, str] and Union[int, str]. Also, some union
        # types can be simplified to non-union types such as Union[int, bool]
        # -> int. It would be nice if we always had simplified union types but
        # this is currently not the case, though it often is.
        left = simplify_union(left)
        right = simplify_union(right)

        return left.accept(SameTypeVisitor(right))


def simplify_union(t: Type) -> Type:
    if isinstance(t, UnionType):
        return UnionType.make_simplified_union(t.items)
    return t


def is_same_types(a1: Sequence[Type], a2: Sequence[Type]) -> bool:
    if len(a1) != len(a2):
        return False
    for i in range(len(a1)):
        if not is_same_type(a1[i], a2[i]):
            return False
    return True


class SameTypeVisitor(TypeVisitor[bool]):
    """Visitor for checking whether two types are the 'same' type."""

    def __init__(self, right: Type) -> None:
        self.right = right

    # visit_x(left) means: is left (which is an instance of X) the same type as
    # right?

    def visit_unbound_type(self, left: UnboundType) -> bool:
        return True

    def visit_any(self, left: AnyType) -> bool:
        return isinstance(self.right, AnyType)

    def visit_none_type(self, left: NoneTyp) -> bool:
        return isinstance(self.right, NoneTyp)

    def visit_uninhabited_type(self, t: UninhabitedType) -> bool:
        return isinstance(self.right, UninhabitedType)

    def visit_erased_type(self, left: ErasedType) -> bool:
        # We can get here when isinstance is used inside a lambda
        # whose type is being inferred. In any event, we have no reason
        # to think that an ErasedType will end up being the same as
        # any other type, except another ErasedType (for protocols).
        return isinstance(self.right, ErasedType)

    def visit_deleted_type(self, left: DeletedType) -> bool:
        return isinstance(self.right, DeletedType)

    def visit_instance(self, left: Instance) -> bool:
        return (isinstance(self.right, Instance) and
                left.type == self.right.type and
                is_same_types(left.args, self.right.args))

    def visit_type_var(self, left: TypeVarType) -> bool:
        return (isinstance(self.right, TypeVarType) and
                left.id == self.right.id)

    def visit_callable_type(self, left: CallableType) -> bool:
        # FIX generics
        if isinstance(self.right, CallableType):
            cright = self.right
            return (is_same_type(left.ret_type, cright.ret_type) and
                    is_same_types(left.arg_types, cright.arg_types) and
                    left.arg_names == cright.arg_names and
                    left.arg_kinds == cright.arg_kinds and
                    left.is_type_obj() == cright.is_type_obj() and
                    left.is_ellipsis_args == cright.is_ellipsis_args)
        else:
            return False

    def visit_tuple_type(self, left: TupleType) -> bool:
        if isinstance(self.right, TupleType):
            return is_same_types(left.items, self.right.items)
        else:
            return False

    def visit_typeddict_type(self, left: TypedDictType) -> bool:
        if isinstance(self.right, TypedDictType):
            if left.items.keys() != self.right.items.keys():
                return False
            for (_, left_item_type, right_item_type) in left.zip(self.right):
                if not is_same_type(left_item_type, right_item_type):
                    return False
            return True
        else:
            return False

    def visit_union_type(self, left: UnionType) -> bool:
        if isinstance(self.right, UnionType):
            # Check that everything in left is in right
            for left_item in left.items:
                if not any(is_same_type(left_item, right_item) for right_item in self.right.items):
                    return False

            # Check that everything in right is in left
            for right_item in self.right.items:
                if not any(is_same_type(right_item, left_item) for left_item in left.items):
                    return False

            return True
        else:
            return False

    def visit_overloaded(self, left: Overloaded) -> bool:
        if isinstance(self.right, Overloaded):
            return is_same_types(left.items(), self.right.items())
        else:
            return False

    def visit_partial_type(self, left: PartialType) -> bool:
        # A partial type is not fully defined, so the result is indeterminate. We shouldn't
        # get here.
        raise RuntimeError

    def visit_type_type(self, left: TypeType) -> bool:
        if isinstance(self.right, TypeType):
            return is_same_type(left.item, self.right.item)
        else:
            return False



from typing import Optional

"""Shared logic between our three mypy parser files."""


_NON_BINARY_MAGIC_METHODS = {
    "__abs__",
    "__call__",
    "__complex__",
    "__contains__",
    "__del__",
    "__delattr__",
    "__delitem__",
    "__enter__",
    "__exit__",
    "__float__",
    "__getattr__",
    "__getattribute__",
    "__getitem__",
    "__hex__",
    "__init__",
    "__init_subclass__",
    "__int__",
    "__invert__",
    "__iter__",
    "__len__",
    "__long__",
    "__neg__",
    "__new__",
    "__nonzero__",
    "__oct__",
    "__pos__",
    "__repr__",
    "__reversed__",
    "__setattr__",
    "__setitem__",
    "__str__",
    "__unicode__",
}

MAGIC_METHODS_ALLOWING_KWARGS = {
    "__init__",
    "__init_subclass__",
    "__new__",
    "__call__",
}

BINARY_MAGIC_METHODS = {
    "__add__",
    "__and__",
    "__cmp__",
    "__divmod__",
    "__div__",
    "__eq__",
    "__floordiv__",
    "__ge__",
    "__gt__",
    "__iadd__",
    "__iand__",
    "__idiv__",
    "__ifloordiv__",
    "__ilshift__",
    "__imod__",
    "__imul__",
    "__ior__",
    "__ipow__",
    "__irshift__",
    "__isub__",
    "__ixor__",
    "__le__",
    "__lshift__",
    "__lt__",
    "__mod__",
    "__mul__",
    "__ne__",
    "__or__",
    "__pow__",
    "__radd__",
    "__rand__",
    "__rdiv__",
    "__rfloordiv__",
    "__rlshift__",
    "__rmod__",
    "__rmul__",
    "__ror__",
    "__rpow__",
    "__rrshift__",
    "__rshift__",
    "__rsub__",
    "__rxor__",
    "__sub__",
    "__xor__",
}

assert not (_NON_BINARY_MAGIC_METHODS & BINARY_MAGIC_METHODS)

MAGIC_METHODS = _NON_BINARY_MAGIC_METHODS | BINARY_MAGIC_METHODS

MAGIC_METHODS_POS_ARGS_ONLY = MAGIC_METHODS - MAGIC_METHODS_ALLOWING_KWARGS


def special_function_elide_names(name: str) -> bool:
    return name in MAGIC_METHODS_POS_ARGS_ONLY


def argument_elide_name(name: Optional[str]) -> bool:
    return name is not None and name.startswith("__") and not name.endswith("__")



from typing import Any, Union, Tuple, Optional, Dict, NoReturn, Sequence

CREATE_NEW_CONSOLE: int
CREATE_NEW_PROCESS_GROUP: int
DUPLICATE_CLOSE_SOURCE: int
DUPLICATE_SAME_ACCESS: int
ERROR_ALREADY_EXISTS: int
ERROR_BROKEN_PIPE: int
ERROR_IO_PENDING: int
ERROR_MORE_DATA: int
ERROR_NETNAME_DELETED: int
ERROR_NO_DATA: int
ERROR_NO_SYSTEM_RESOURCES: int
ERROR_OPERATION_ABORTED: int
ERROR_PIPE_BUSY: int
ERROR_PIPE_CONNECTED: int
ERROR_SEM_TIMEOUT: int
FILE_FLAG_FIRST_PIPE_INSTANCE: int
FILE_FLAG_OVERLAPPED: int
FILE_GENERIC_READ: int
FILE_GENERIC_WRITE: int
GENERIC_READ: int
GENERIC_WRITE: int
INFINITE: int
NMPWAIT_WAIT_FOREVER: int
NULL: int
OPEN_EXISTING: int
PIPE_ACCESS_DUPLEX: int
PIPE_ACCESS_INBOUND: int
PIPE_READMODE_MESSAGE: int
PIPE_TYPE_MESSAGE: int
PIPE_UNLIMITED_INSTANCES: int
PIPE_WAIT: int
PROCESS_ALL_ACCESS: int
PROCESS_DUP_HANDLE: int
STARTF_USESHOWWINDOW: int
STARTF_USESTDHANDLES: int
STD_ERROR_HANDLE: int
STD_INPUT_HANDLE: int
STD_OUTPUT_HANDLE: int
STILL_ACTIVE: int
SW_HIDE: int
WAIT_ABANDONED_0: int
WAIT_OBJECT_0: int
WAIT_TIMEOUT: int

def CloseHandle(handle: int) -> None: ...
def ConnectNamedPipe(handle: int, overlapped: Union[int, bool] = ...) -> None: ...
def CreateFile(file_name: str, desired_access: int, share_mode: int, security_attributes: int, creation_disposition: int, flags_and_attributes: int, template_file: int) -> int: ...
def CreateJunction(src_path: str, dest_path: str) -> None: ...
def CreateNamedPipe(name: str, open_mode: int, pipe_mode: int, max_instances: int, out_buffer_size: int, in_buffer_size: int, default_timeout: int, security_attributes: int) -> int: ...
def CreatePipe(pipe_attrs: Any, size: int) -> Tuple[int, int]: ...
def CreateProcess(application_name: Optional[str], command_line: Optional[str], proc_attrs: Any, thread_attrs: Any, inherit_handles: bool, creation_flags: int, env_mapping: Dict[str, str], cwd: Optional[str], startup_info: Any) -> Tuple[int, int, int, int]: ...
def DuplicateHandle(source_process_handle: int, source_handle: int, target_process_handle: int, desired_access: int, inherit_handle: bool, options: int = ...) -> int: ...
def ExitProcess(ExitCode: int) -> NoReturn: ...
def GetACP() -> int: ...
def GetFileType(handle: int) -> int: ...
def GetCurrentProcess() -> int: ...
def GetExitCodeProcess(process: int) -> int: ...
def GetLastError() -> int: ...
def GetModuleFileName(module_handle: int) -> str: ...
def GetStdHandle(std_handle: int) -> int: ...
def GetVersion() -> int: ...
def OpenProcess(desired_access: int, inherit_handle: bool, process_id: int) -> int: ...
def PeekNamedPipe(handle: int, size: int = ...) -> Union[Tuple[int, int], Tuple[bytes, int, int]]: ...
def ReadFile(handle: int, size: int, overlapped: Union[int, bool] = ...) -> Tuple[bytes, int]: ...
def SetNamedPipeHandleState(named_pipe: int, mode: int, max_collection_count: int, collect_data_timeout: int) -> None: ...
def TerminateProcess(handle: int, exit_code: int) -> None: ...
def WaitForMultipleObjects(handle_seq: Sequence[int], wait_flag: bool, milliseconds: int = ...) -> int: ...
def WaitForSingleObject(handle: int, milliseconds: int) -> int: ...
def WaitNamedPipe(name: str, timeout: int) -> None: ...
def WriteFile(handle: int, buffer: bytes, overlapped: Union[int, bool] = ...): ...

class Overlapped:
    event: int = ...
    def GetOverlappedResult(self, wait: bool) -> Tuple[int, int]: ...
    def cancel(self) -> None: ...
    def getbuffer(self) -> Optional[bytes]: ...



from typing import Optional, Any
from ._base import Future, Executor
import sys

EXTRA_QUEUED_CALLS = ...  # type: Any

if sys.version_info >= (3,):
    class BrokenProcessPool(RuntimeError): ...

class ProcessPoolExecutor(Executor):
    def __init__(self, max_workers: Optional[int] = ...) -> None: ...



import importlib.abc
import sys
import types
from typing import Any, Callable, List, Optional, Sequence, Tuple, Union

# ModuleSpec is exported from this module, but for circular import
# reasons exists in its own stub file (with Loader and ModuleType).
from _importlib_modulespec import ModuleSpec as ModuleSpec  # Exported

class BuiltinImporter(importlib.abc.MetaPathFinder,
                      importlib.abc.InspectLoader):
    # MetaPathFinder
    @classmethod
    def find_module(
        cls, fullname: str,
        path: Optional[Sequence[importlib.abc._Path]]
    ) -> Optional[importlib.abc.Loader]:
        ...
    @classmethod
    def find_spec(cls, fullname: str,
                  path: Optional[Sequence[importlib.abc._Path]],
                  target: Optional[types.ModuleType] = ...) -> Optional[ModuleSpec]:
            ...
    # InspectLoader
    @classmethod
    def is_package(cls, fullname: str) -> bool: ...
    @classmethod
    def load_module(cls, fullname: str) -> types.ModuleType: ...
    @classmethod
    def get_code(cls, fullname: str) -> None: ...
    @classmethod
    def get_source(cls, fullname: str) -> None: ...
    # Loader
    @classmethod
    def load_module(cls, fullname: str) -> types.ModuleType: ...
    @staticmethod
    def module_repr(module: types.ModuleType) -> str: ...  # type: ignore
    @classmethod
    def create_module(cls, spec: ModuleSpec) -> Optional[types.ModuleType]: ...
    @classmethod
    def exec_module(cls, module: types.ModuleType) -> None: ...

class FrozenImporter(importlib.abc.MetaPathFinder, importlib.abc.InspectLoader):
    # MetaPathFinder
    @classmethod
    def find_module(
        cls, fullname: str,
        path: Optional[Sequence[importlib.abc._Path]]
    ) -> Optional[importlib.abc.Loader]:
        ...
    @classmethod
    def find_spec(cls, fullname: str,
                  path: Optional[Sequence[importlib.abc._Path]],
                  target: Optional[types.ModuleType] = ...) -> Optional[ModuleSpec]:
        ...
    # InspectLoader
    @classmethod
    def is_package(cls, fullname: str) -> bool: ...
    @classmethod
    def load_module(cls, fullname: str) -> types.ModuleType: ...
    @classmethod
    def get_code(cls, fullname: str) -> None: ...
    @classmethod
    def get_source(cls, fullname: str) -> None: ...
    # Loader
    @classmethod
    def load_module(cls, fullname: str) -> types.ModuleType: ...
    @staticmethod
    def module_repr(module: types.ModuleType) -> str: ...  # type: ignore
    @classmethod
    def create_module(cls, spec: ModuleSpec) -> Optional[types.ModuleType]:
        ...
    @staticmethod
    def exec_module(module: types.ModuleType) -> None: ...  # type: ignore

class WindowsRegistryFinder(importlib.abc.MetaPathFinder):
    @classmethod
    def find_module(
        cls, fullname: str,
        path: Optional[Sequence[importlib.abc._Path]]
    ) -> Optional[importlib.abc.Loader]:
        ...
    @classmethod
    def find_spec(cls, fullname: str,
                  path: Optional[Sequence[importlib.abc._Path]],
                  target: Optional[types.ModuleType] = ...) -> Optional[ModuleSpec]:
        ...

class PathFinder(importlib.abc.MetaPathFinder): ...

SOURCE_SUFFIXES = ...  # type: List[str]
DEBUG_BYTECODE_SUFFIXES = ...  # type: List[str]
OPTIMIZED_BYTECODE_SUFFIXES = ...  # type: List[str]
BYTECODE_SUFFIXES = ...  # type: List[str]
EXTENSION_SUFFIXES = ...  # type: List[str]

def all_suffixes() -> List[str]: ...

class FileFinder(importlib.abc.PathEntryFinder):
    path = ...  # type: str
    def __init__(
        self, path: str,
        *loader_details: Tuple[importlib.abc.Loader, List[str]]
    ) -> None: ...
    @classmethod
    def path_hook(
        cls, *loader_details: Tuple[importlib.abc.Loader, List[str]]
    ) -> Callable[[str], importlib.abc.PathEntryFinder]: ...

class SourceFileLoader(importlib.abc.FileLoader,
                       importlib.abc.SourceLoader):
    ...

class SourcelessFileLoader(importlib.abc.FileLoader,
                           importlib.abc.SourceLoader):
    ...

class ExtensionFileLoader(importlib.abc.ExecutionLoader):
    def get_filename(self, fullname: str) -> importlib.abc._Path: ...
    def get_source(self, fullname: str) -> None: ...  # type: ignore



"""Find all subexpressions of an AST node."""

from typing import List

from mypy.nodes import (
    Expression, Node, MemberExpr, YieldFromExpr, YieldExpr, CallExpr, OpExpr, ComparisonExpr,
    SliceExpr, CastExpr, RevealExpr, UnaryExpr, ListExpr, TupleExpr, DictExpr, SetExpr,
    IndexExpr, GeneratorExpr, ListComprehension, SetComprehension, DictionaryComprehension,
    ConditionalExpr, TypeApplication, LambdaExpr, StarExpr, BackquoteExpr, AwaitExpr,
)
from mypy.traverser import TraverserVisitor


def get_subexpressions(node: Node) -> List[Expression]:
    visitor = SubexpressionFinder()
    node.accept(visitor)
    return visitor.expressions


class SubexpressionFinder(TraverserVisitor):
    def __init__(self) -> None:
        self.expressions = []  # type: List[Expression]

    def _visit_leaf(self, o: Expression) -> None:
        self.add(o)

    visit_int_expr = _visit_leaf
    visit_name_expr = _visit_leaf
    visit_float_expr = _visit_leaf
    visit_str_expr = _visit_leaf
    visit_bytes_expr = _visit_leaf
    visit_unicode_expr = _visit_leaf
    visit_complex_expr = _visit_leaf
    visit_ellipsis = _visit_leaf
    visit_super_expr = _visit_leaf
    visit_type_var_expr = _visit_leaf
    visit_type_alias_expr = _visit_leaf
    visit_namedtuple_expr = _visit_leaf
    visit_typeddict_expr = _visit_leaf
    visit__promote_expr = _visit_leaf
    visit_newtype_expr = _visit_leaf

    def visit_member_expr(self, e: MemberExpr) -> None:
        self.add(e)
        super().visit_member_expr(e)

    def visit_yield_from_expr(self, e: YieldFromExpr) -> None:
        self.add(e)
        super().visit_yield_from_expr(e)

    def visit_yield_expr(self, e: YieldExpr) -> None:
        self.add(e)
        super().visit_yield_expr(e)

    def visit_call_expr(self, e: CallExpr) -> None:
        self.add(e)
        super().visit_call_expr(e)

    def visit_op_expr(self, e: OpExpr) -> None:
        self.add(e)
        super().visit_op_expr(e)

    def visit_comparison_expr(self, e: ComparisonExpr) -> None:
        self.add(e)
        super().visit_comparison_expr(e)

    def visit_slice_expr(self, e: SliceExpr) -> None:
        self.add(e)
        super().visit_slice_expr(e)

    def visit_cast_expr(self, e: CastExpr) -> None:
        self.add(e)
        super().visit_cast_expr(e)

    def visit_reveal_expr(self, e: RevealExpr) -> None:
        self.add(e)
        super().visit_reveal_expr(e)

    def visit_unary_expr(self, e: UnaryExpr) -> None:
        self.add(e)
        super().visit_unary_expr(e)

    def visit_list_expr(self, e: ListExpr) -> None:
        self.add(e)
        super().visit_list_expr(e)

    def visit_tuple_expr(self, e: TupleExpr) -> None:
        self.add(e)
        super().visit_tuple_expr(e)

    def visit_dict_expr(self, e: DictExpr) -> None:
        self.add(e)
        super().visit_dict_expr(e)

    def visit_set_expr(self, e: SetExpr) -> None:
        self.add(e)
        super().visit_set_expr(e)

    def visit_index_expr(self, e: IndexExpr) -> None:
        self.add(e)
        super().visit_index_expr(e)

    def visit_generator_expr(self, e: GeneratorExpr) -> None:
        self.add(e)
        super().visit_generator_expr(e)

    def visit_dictionary_comprehension(self, e: DictionaryComprehension) -> None:
        self.add(e)
        super().visit_dictionary_comprehension(e)

    def visit_list_comprehension(self, e: ListComprehension) -> None:
        self.add(e)
        super().visit_list_comprehension(e)

    def visit_set_comprehension(self, e: SetComprehension) -> None:
        self.add(e)
        super().visit_set_comprehension(e)

    def visit_conditional_expr(self, e: ConditionalExpr) -> None:
        self.add(e)
        super().visit_conditional_expr(e)

    def visit_type_application(self, e: TypeApplication) -> None:
        self.add(e)
        super().visit_type_application(e)

    def visit_lambda_expr(self, e: LambdaExpr) -> None:
        self.add(e)
        super().visit_lambda_expr(e)

    def visit_star_expr(self, e: StarExpr) -> None:
        self.add(e)
        super().visit_star_expr(e)

    def visit_backquote_expr(self, e: BackquoteExpr) -> None:
        self.add(e)
        super().visit_backquote_expr(e)

    def visit_await_expr(self, e: AwaitExpr) -> None:
        self.add(e)
        super().visit_await_expr(e)

    def add(self, e: Expression) -> None:
        self.expressions.append(e)



from typing import Any, Dict, Set, Tuple, Type
import _weakrefset

# NOTE: mypy has special processing for ABCMeta and abstractmethod.

def abstractmethod(funcobj: Any) -> Any: ...

class ABCMeta(type):
    # TODO: FrozenSet
    __abstractmethods__ = ...  # type: Set[Any]
    _abc_cache = ...  # type: _weakrefset.WeakSet
    _abc_invalidation_counter = ...  # type: int
    _abc_negative_cache = ...  # type: _weakrefset.WeakSet
    _abc_negative_cache_version = ...  # type: int
    _abc_registry = ...  # type: _weakrefset.WeakSet
    def __init__(self, name: str, bases: Tuple[type, ...], namespace: Dict[Any, Any]) -> None: ...
    def __instancecheck__(cls: ABCMeta, instance: Any) -> Any: ...
    def __subclasscheck__(cls: ABCMeta, subclass: Any) -> Any: ...
    def _dump_registry(cls: ABCMeta, *args: Any, **kwargs: Any) -> None: ...
    def register(cls: ABCMeta, subclass: Type[Any]) -> None: ...

# TODO: The real abc.abstractproperty inherits from "property".
class abstractproperty(object):
    def __new__(cls, func: Any) -> Any: ...
    __isabstractmethod__ = ...  # type: bool
    doc = ...  # type: Any
    fdel = ...  # type: Any
    fget = ...  # type: Any
    fset = ...  # type: Any



"""Utility functions with no non-trivial dependencies."""
import genericpath  # type: ignore  # no stub files yet
import os
import pathlib
import re
import subprocess
import sys
from xml.sax.saxutils import escape
from typing import TypeVar, List, Tuple, Optional, Dict, Sequence


T = TypeVar('T')

ENCODING_RE = re.compile(br'([ \t\v]*#.*(\r\n?|\n))??[ \t\v]*#.*coding[:=][ \t]*([-\w.]+)')

default_python2_interpreter = ['python2', 'python', '/usr/bin/python', 'C:\\Python27\\python.exe']


def split_module_names(mod_name: str) -> List[str]:
    """Return the module and all parent module names.

    So, if `mod_name` is 'a.b.c', this function will return
    ['a.b.c', 'a.b', and 'a'].
    """
    out = [mod_name]
    while '.' in mod_name:
        mod_name = mod_name.rsplit('.', 1)[0]
        out.append(mod_name)
    return out


def short_type(obj: object) -> str:
    """Return the last component of the type name of an object.

    If obj is None, return 'nil'. For example, if obj is 1, return 'int'.
    """
    if obj is None:
        return 'nil'
    t = str(type(obj))
    return t.split('.')[-1].rstrip("'>")


def array_repr(a: List[T]) -> List[str]:
    """Return the items of an array converted to strings using Repr."""
    aa = []  # type: List[str]
    for x in a:
        aa.append(repr(x))
    return aa


def find_python_encoding(text: bytes, pyversion: Tuple[int, int]) -> Tuple[str, int]:
    """PEP-263 for detecting Python file encoding"""
    result = ENCODING_RE.match(text)
    if result:
        line = 2 if result.group(1) else 1
        encoding = result.group(3).decode('ascii')
        # Handle some aliases that Python is happy to accept and that are used in the wild.
        if encoding.startswith(('iso-latin-1-', 'latin-1-')) or encoding == 'iso-latin-1':
            encoding = 'latin-1'
        return encoding, line
    else:
        default_encoding = 'utf8' if pyversion[0] >= 3 else 'ascii'
        return default_encoding, -1


class DecodeError(Exception):
    """Exception raised when a file cannot be decoded due to an unknown encoding type.

    Essentially a wrapper for the LookupError raised by `bytearray.decode`
    """


def decode_python_encoding(source: bytes, pyversion: Tuple[int, int]) -> str:
    """Read the Python file with while obeying PEP-263 encoding detection.

    Returns:
      A tuple: the source as a string, and the hash calculated from the binary representation.
    """
    # check for BOM UTF-8 encoding and strip it out if present
    if source.startswith(b'\xef\xbb\xbf'):
        encoding = 'utf8'
        source = source[3:]
    else:
        # look at first two lines and check if PEP-263 coding is present
        encoding, _ = find_python_encoding(source, pyversion)

    try:
        source_text = source.decode(encoding)
    except LookupError as lookuperr:
        raise DecodeError(str(lookuperr))
    return source_text


_python2_interpreter = None  # type: Optional[str]


def try_find_python2_interpreter() -> Optional[str]:
    global _python2_interpreter
    if _python2_interpreter:
        return _python2_interpreter
    for interpreter in default_python2_interpreter:
        try:
            retcode = subprocess.Popen([
                interpreter, '-c',
                'import sys, typing; assert sys.version_info[:2] == (2, 7)'
            ]).wait()
            if not retcode:
                _python2_interpreter = interpreter
                return interpreter
        except OSError:
            pass
    return None


PASS_TEMPLATE = """<?xml version="1.0" encoding="utf-8"?>
<testsuite errors="0" failures="0" name="mypy" skips="0" tests="1" time="{time:.3f}">
  <testcase classname="mypy" file="mypy" line="1" name="mypy" time="{time:.3f}">
  </testcase>
</testsuite>
"""

FAIL_TEMPLATE = """<?xml version="1.0" encoding="utf-8"?>
<testsuite errors="0" failures="1" name="mypy" skips="0" tests="1" time="{time:.3f}">
  <testcase classname="mypy" file="mypy" line="1" name="mypy" time="{time:.3f}">
    <failure message="mypy produced messages">{text}</failure>
  </testcase>
</testsuite>
"""

ERROR_TEMPLATE = """<?xml version="1.0" encoding="utf-8"?>
<testsuite errors="1" failures="0" name="mypy" skips="0" tests="1" time="{time:.3f}">
  <testcase classname="mypy" file="mypy" line="1" name="mypy" time="{time:.3f}">
    <error message="mypy produced errors">{text}</error>
  </testcase>
</testsuite>
"""


def write_junit_xml(dt: float, serious: bool, messages: List[str], path: str) -> None:
    """XXX"""
    if not messages and not serious:
        xml = PASS_TEMPLATE.format(time=dt)
    elif not serious:
        xml = FAIL_TEMPLATE.format(text=escape('\n'.join(messages)), time=dt)
    else:
        xml = ERROR_TEMPLATE.format(text=escape('\n'.join(messages)), time=dt)
    with open(path, 'wb') as f:
        f.write(xml.encode('utf-8'))


class IdMapper:
    """Generate integer ids for objects.

    Unlike id(), these start from 0 and increment by 1, and ids won't
    get reused across the life-time of IdMapper.

    Assume objects don't redefine __eq__ or __hash__.
    """

    def __init__(self) -> None:
        self.id_map = {}  # type: Dict[object, int]
        self.next_id = 0

    def id(self, o: object) -> int:
        if o not in self.id_map:
            self.id_map[o] = self.next_id
            self.next_id += 1
        return self.id_map[o]


def get_prefix(fullname: str) -> str:
    """Drop the final component of a qualified name (e.g. ('x.y' -> 'x')."""
    return fullname.rsplit('.', 1)[0]


def correct_relative_import(cur_mod_id: str,
                            relative: int,
                            target: str,
                            is_cur_package_init_file: bool) -> Tuple[str, bool]:
    if relative == 0:
        return target, True
    parts = cur_mod_id.split(".")
    rel = relative
    if is_cur_package_init_file:
        rel -= 1
    ok = len(parts) >= rel
    if rel != 0:
        cur_mod_id = ".".join(parts[:-rel])
    return cur_mod_id + (("." + target) if target else ""), ok


def replace_object_state(new: object, old: object) -> None:
    """Copy state of old node to the new node.

    This handles cases where there is __slots__ and/or __dict__.

    Assume that both objects have the same __class__.
    """
    if hasattr(old, '__dict__'):
        new.__dict__ = old.__dict__
    if hasattr(old, '__slots__'):
        # Use __mro__ since some classes override 'mro' with something different.
        for base in type(old).__mro__:
            if '__slots__' in base.__dict__:
                for attr in getattr(base, '__slots__'):
                    if hasattr(old, attr):
                        setattr(new, attr, getattr(old, attr))
                    elif hasattr(new, attr):
                        delattr(new, attr)


def is_sub_path(path1: str, path2: str) -> bool:
    """Given two paths, return if path1 is a sub-path of path2."""
    return pathlib.Path(path2) in pathlib.Path(path1).parents



# Stubs for stat

# Based on http://docs.python.org/3.2/library/stat.html

import sys
import typing

def S_ISDIR(mode: int) -> bool: ...
def S_ISCHR(mode: int) -> bool: ...
def S_ISBLK(mode: int) -> bool: ...
def S_ISREG(mode: int) -> bool: ...
def S_ISFIFO(mode: int) -> bool: ...
def S_ISLNK(mode: int) -> bool: ...
def S_ISSOCK(mode: int) -> bool: ...

def S_IMODE(mode: int) -> int: ...
def S_IFMT(mode: int) -> int: ...

def filemode(mode: int) -> str: ...

ST_MODE = 0
ST_INO = 0
ST_DEV = 0
ST_NLINK = 0
ST_UID = 0
ST_GID = 0
ST_SIZE = 0
ST_ATIME = 0
ST_MTIME = 0
ST_CTIME = 0

S_IFSOCK = 0
S_IFLNK = 0
S_IFREG = 0
S_IFBLK = 0
S_IFDIR = 0
S_IFCHR = 0
S_IFIFO = 0
S_ISUID = 0
S_ISGID = 0
S_ISVTX = 0

S_IRWXU = 0
S_IRUSR = 0
S_IWUSR = 0
S_IXUSR = 0

S_IRWXG = 0
S_IRGRP = 0
S_IWGRP = 0
S_IXGRP = 0

S_IRWXO = 0
S_IROTH = 0
S_IWOTH = 0
S_IXOTH = 0

S_ENFMT = 0
S_IREAD = 0
S_IWRITE = 0
S_IEXEC = 0

UF_NODUMP = 0
UF_IMMUTABLE = 0
UF_APPEND = 0
UF_OPAQUE = 0
UF_NOUNLINK = 0
if sys.platform == 'darwin':
    UF_COMPRESSED = 0  # OS X 10.6+ only
    UF_HIDDEN = 0     # OX X 10.5+ only
SF_ARCHIVED = 0
SF_IMMUTABLE = 0
SF_APPEND = 0
SF_NOUNLINK = 0
SF_SNAPSHOT = 0



# Stubs for posix

# NOTE: These are incomplete!

from typing import NamedTuple, Tuple

class stat_result:
    # For backward compatibility, the return value of stat() is also
    # accessible as a tuple of at least 10 integers giving the most important
    # (and portable) members of the stat structure, in the order st_mode,
    # st_ino, st_dev, st_nlink, st_uid, st_gid, st_size, st_atime, st_mtime,
    # st_ctime. More items may be added at the end by some implementations.

    st_mode: int  # protection bits,
    st_ino: int  # inode number,
    st_dev: int  # device,
    st_nlink: int  # number of hard links,
    st_uid: int  # user id of owner,
    st_gid: int  # group id of owner,
    st_size: int  # size of file, in bytes,
    st_atime: float  # time of most recent access,
    st_mtime: float  # time of most recent content modification,
    st_ctime: float  # platform dependent (time of most recent metadata change on Unix, or the time of creation on Windows)
    st_atime_ns: int  # time of most recent access, in nanoseconds
    st_mtime_ns: int  # time of most recent content modification in nanoseconds
    st_ctime_ns: int  # platform dependent (time of most recent metadata change on Unix, or the time of creation on Windows) in nanoseconds

    # not documented
    def __init__(self, tuple: Tuple[int, ...]) -> None: ...

    # On some Unix systems (such as Linux), the following attributes may also
    # be available:
    st_blocks: int  # number of blocks allocated for file
    st_blksize: int  # filesystem blocksize
    st_rdev: int  # type of device if an inode device
    st_flags: int  # user defined flags for file

    # On other Unix systems (such as FreeBSD), the following attributes may be
    # available (but may be only filled out if root tries to use them):
    st_gen: int  # file generation number
    st_birthtime: int  # time of file creation

    # On Mac OS systems, the following attributes may also be available:
    st_rsize: int
    st_creator: int
    st_type: int

uname_result = NamedTuple('uname_result', [('sysname', str), ('nodename', str),
    ('release', str), ('version', str), ('machine', str)])

times_result = NamedTuple('times_result', [
    ('user', float),
    ('system', float),
    ('children_user', float),
    ('children_system', float),
    ('elapsed', float),
])

waitid_result = NamedTuple('waitid_result', [
    ('si_pid', int),
    ('si_uid', int),
    ('si_signo', int),
    ('si_status', int),
    ('si_code', int),
])

sched_param = NamedTuple('sched_priority', [
    ('sched_priority', int),
])



import sys

from typing import Any, Iterable, Iterator, List, Optional, Sequence

QUOTE_ALL = ...  # type: int
QUOTE_MINIMAL = ...  # type: int
QUOTE_NONE = ...  # type: int
QUOTE_NONNUMERIC = ...  # type: int

class Error(Exception): ...

class Dialect:
    delimiter = ...  # type: str
    quotechar = ...  # type: Optional[str]
    escapechar = ...  # type: Optional[str]
    doublequote = ...  # type: bool
    skipinitialspace = ...  # type: bool
    lineterminator = ...  # type: str
    quoting = ...  # type: int
    strict = ...  # type: int
    def __init__(self) -> None: ...

class _reader(Iterator[List[str]]):
    dialect = ...  # type: Dialect
    line_num = ...  # type: int
    if sys.version_info >= (3, 0):
        def __next__(self) -> List[str]: ...
    else:
        def next(self) -> List[str]: ...

class _writer:
    dialect = ...  # type: Dialect

    if sys.version_info >= (3, 5):
        def writerow(self, row: Iterable[Any]) -> None: ...
        def writerows(self, rows: Iterable[Iterable[Any]]) -> None: ...
    else:
        def writerow(self, row: Sequence[Any]) -> None: ...
        def writerows(self, rows: Iterable[Sequence[Any]]) -> None: ...


# TODO: precise type
def writer(csvfile: Any, dialect: Any = ..., **fmtparams: Any) -> _writer: ...
def reader(csvfile: Iterable[str], dialect: Any = ..., **fmtparams: Any) -> _reader: ...
def register_dialect(name: str, dialect: Any = ..., **fmtparams: Any) -> None: ...
def unregister_dialect(name: str) -> None: ...
def get_dialect(name: str) -> Dialect: ...
def list_dialects() -> List[str]: ...
def field_size_limit(new_limit: int = ...) -> int: ...



from typing import Any

HTTP_STATUS_CODES = ...  # type: Any

def wsgi_to_bytes(data): ...
def bytes_to_wsgi(data): ...
def quote_header_value(value, extra_chars='', allow_token=True): ...
def unquote_header_value(value, is_filename=False): ...
def dump_options_header(header, options): ...
def dump_header(iterable, allow_token=True): ...
def parse_list_header(value): ...
def parse_dict_header(value, cls=...): ...
def parse_options_header(value, multiple=False): ...
def parse_accept_header(value, cls=None): ...
def parse_cache_control_header(value, on_update=None, cls=None): ...
def parse_set_header(value, on_update=None): ...
def parse_authorization_header(value): ...
def parse_www_authenticate_header(value, on_update=None): ...
def parse_if_range_header(value): ...
def parse_range_header(value, make_inclusive=True): ...
def parse_content_range_header(value, on_update=None): ...
def quote_etag(etag, weak=False): ...
def unquote_etag(etag): ...
def parse_etags(value): ...
def generate_etag(data): ...
def parse_date(value): ...
def cookie_date(expires=None): ...
def http_date(timestamp=None): ...
def is_resource_modified(environ, etag=None, data=None, last_modified=None, ignore_if_range=True): ...
def remove_entity_headers(headers, allowed=...): ...
def remove_hop_by_hop_headers(headers): ...
def is_entity_header(header): ...
def is_hop_by_hop_header(header): ...
def parse_cookie(header, charset='', errors='', cls=None): ...
def dump_cookie(key, value='', max_age=None, expires=None, path='', domain=None, secure=False, httponly=False, charset='', sync_expires=True): ...
def is_byte_range_valid(start, stop, length): ...



# NB: posixpath.pyi, ntpath.pyi, and macpath.pyi must remain consistent!
# Stubs for os.path
# Ron Murawski <ron@horizonchess.com>

# based on http://docs.python.org/3.2/library/os.path.html
import sys
from typing import Any, List, Tuple, IO

# ----- os.path variables -----
supports_unicode_filenames = False

# ----- os.path function stubs -----
def abspath(path: str) -> str: ...
def basename(path) -> str: ...
def commonprefix(list: List[str]) -> str: ...
def dirname(path: str) -> str: ...
def exists(path: str) -> bool: ...
def lexists(path: str) -> bool: ...
def expanduser(path: str) -> str: ...
def expandvars(path: str) -> str: ...
def getatime(path: str) -> int:
    ...  # return float if os.stat_float_times() returns True
def getmtime(path: str) -> int:
    ...  # return float if os.stat_float_times() returns True
def getctime(path: str) -> int:
    ...  # return float if os.stat_float_times() returns True
def getsize(path: str) -> int: ...
def isabs(path: str) -> bool: ...
def isfile(path: str) -> bool: ...
def isdir(path: str) -> bool: ...
def islink(path: str) -> bool: ...
def ismount(path: str) -> bool: ...
def join(path: str, *paths: str) -> str: ...
def normcase(path: str) -> str: ...
def normpath(path: str) -> str: ...
def realpath(path: str) -> str: ...
def relpath(path: str, start: str = ...) -> str: ...
def samefile(path1: str, path2: str) -> bool: ...

def sameopenfile(fp1: IO[Any], fp2: IO[Any]) -> bool: ...

# def samestat(stat1: stat_result, stat2: stat_result) -> bool:
#    ...  # Unix only
def split(path: str) -> Tuple[str, str]: ...
def splitdrive(path: str) -> Tuple[str, str]: ...
def splitext(path: str) -> Tuple[str, str]: ...
if sys.version_info < (3, 7) and sys.platform == 'win32':
    def splitunc(path: str) -> Tuple[str, str]: ...



from typing import Any, Iterable, List, Optional, Tuple, Type, Union
import socket
import types

# https://docs.python.org/3/library/multiprocessing.html#address-formats
_Address = Union[str, Tuple[str, int]]

def deliver_challenge(connection: Connection, authkey: bytes) -> None: ...
def answer_challenge(connection: Connection, authkey: bytes) -> None: ...
def wait(object_list: Iterable[Union[Connection, socket.socket, int]], timeout: Optional[float] = ...) -> List[Union[Connection, socket.socket, int]]: ...
def Client(address: _Address, family: Optional[str] = ..., authkey: Optional[bytes] = ...) -> Connection: ...
def Pipe(duplex: bool = ...) -> Tuple[Connection, Connection]: ...

class Listener:
    def __init__(self, address: Optional[_Address] = ..., family: Optional[str] = ..., backlog: int = ..., authkey: Optional[bytes] = ...) -> None: ...
    def accept(self) -> Connection: ...
    def close(self) -> None: ...
    @property
    def address(self) -> _Address: ...
    @property
    def last_accepted(self) -> Optional[_Address]: ...
    def __enter__(self) -> Listener: ...
    def __exit__(self, exc_type: Optional[Type[BaseException]], exc_value: Optional[BaseException], exc_tb: Optional[types.TracebackType]) -> None: ...

class Connection:
    def close(self) -> None: ...
    def fileno(self) -> int: ...
    def poll(self, timeout: Optional[float] = ...) -> bool: ...
    def recv(self) -> Any: ...
    def recv_bytes(self, maxlength: Optional[int] = ...) -> bytes: ...
    def recv_bytes_into(self, buf: Any, offset: int = ...) -> int: ...
    def send(self, obj: Any) -> None: ...
    def send_bytes(self,
                   buf: bytes,
                   offset: int = ...,
                   size: Optional[int] = ...) -> None: ...



"""Utilities for calculating and reporting statistics about types."""

import os.path
import typing

from collections import Counter
from typing import Dict, List, cast, Optional

from mypy.traverser import TraverserVisitor
from mypy.typeanal import collect_all_inner_types
from mypy.types import (
    Type, AnyType, Instance, FunctionLike, TupleType, TypeVarType, TypeQuery, CallableType,
    TypeOfAny
)
from mypy import nodes
from mypy.nodes import (
    Expression, FuncDef, TypeApplication, AssignmentStmt, NameExpr, CallExpr, MypyFile,
    MemberExpr, OpExpr, ComparisonExpr, IndexExpr, UnaryExpr, YieldFromExpr, RefExpr, ClassDef
)


TYPE_EMPTY = 0
TYPE_UNANALYZED = 1  # type of non-typechecked code
TYPE_PRECISE = 2
TYPE_IMPRECISE = 3
TYPE_ANY = 4

precision_names = [
    'empty',
    'unanalyzed',
    'precise',
    'imprecise',
    'any',
]


class StatisticsVisitor(TraverserVisitor):
    def __init__(self,
                 inferred: bool,
                 filename: str,
                 typemap: Optional[Dict[Expression, Type]] = None,
                 all_nodes: bool = False,
                 visit_untyped_defs: bool = True) -> None:
        self.inferred = inferred
        self.filename = filename
        self.typemap = typemap
        self.all_nodes = all_nodes
        self.visit_untyped_defs = visit_untyped_defs

        self.num_precise_exprs = 0
        self.num_imprecise_exprs = 0
        self.num_any_exprs = 0

        self.num_simple_types = 0
        self.num_generic_types = 0
        self.num_tuple_types = 0
        self.num_function_types = 0
        self.num_typevar_types = 0
        self.num_complex_types = 0
        self.num_any_types = 0

        self.line = -1

        self.line_map = {}  # type: Dict[int, int]

        self.type_of_any_counter = Counter()  # type: typing.Counter[TypeOfAny]
        self.any_line_map = {}  # type: Dict[int, List[AnyType]]

        self.output = []  # type: List[str]

        TraverserVisitor.__init__(self)

    def visit_func_def(self, o: FuncDef) -> None:
        self.line = o.line
        if len(o.expanded) > 1 and o.expanded != [o] * len(o.expanded):
            if o in o.expanded:
                print('{}:{}: ERROR: cycle in function expansion; skipping'.format(self.filename,
                                                                                   o.get_line()))
                return
            for defn in o.expanded:
                self.visit_func_def(cast(FuncDef, defn))
        else:
            if o.type:
                sig = cast(CallableType, o.type)
                arg_types = sig.arg_types
                if (sig.arg_names and sig.arg_names[0] == 'self' and
                        not self.inferred):
                    arg_types = arg_types[1:]
                for arg in arg_types:
                    self.type(arg)
                self.type(sig.ret_type)
            elif self.all_nodes:
                self.record_line(self.line, TYPE_ANY)
            if not o.is_dynamic() or self.visit_untyped_defs:
                super().visit_func_def(o)

    def visit_class_def(self, o: ClassDef) -> None:
        # Override this method because we don't want to analyze base_type_exprs (base_type_exprs
        # are base classes in a class declaration).
        # While base_type_exprs are technically expressions, type analyzer does not visit them and
        # they are not in the typemap.
        for d in o.decorators:
            d.accept(self)
        o.defs.accept(self)

    def visit_type_application(self, o: TypeApplication) -> None:
        self.line = o.line
        for t in o.types:
            self.type(t)
        super().visit_type_application(o)

    def visit_assignment_stmt(self, o: AssignmentStmt) -> None:
        self.line = o.line
        if (isinstance(o.rvalue, nodes.CallExpr) and
                isinstance(o.rvalue.analyzed, nodes.TypeVarExpr)):
            # Type variable definition -- not a real assignment.
            return
        if o.type:
            self.type(o.type)
        elif self.inferred and not self.all_nodes:
            # if self.all_nodes is set, lvalues will be visited later
            for lvalue in o.lvalues:
                if isinstance(lvalue, nodes.TupleExpr):
                    items = lvalue.items
                else:
                    items = [lvalue]
                for item in items:
                    if isinstance(item, RefExpr) and item.is_inferred_def:
                        if self.typemap is not None:
                            self.type(self.typemap.get(item))
        super().visit_assignment_stmt(o)

    def visit_name_expr(self, o: NameExpr) -> None:
        self.process_node(o)
        super().visit_name_expr(o)

    def visit_yield_from_expr(self, o: YieldFromExpr) -> None:
        if o.expr:
            o.expr.accept(self)

    def visit_call_expr(self, o: CallExpr) -> None:
        self.process_node(o)
        if o.analyzed:
            o.analyzed.accept(self)
        else:
            o.callee.accept(self)
            for a in o.args:
                a.accept(self)

    def visit_member_expr(self, o: MemberExpr) -> None:
        self.process_node(o)
        super().visit_member_expr(o)

    def visit_op_expr(self, o: OpExpr) -> None:
        self.process_node(o)
        super().visit_op_expr(o)

    def visit_comparison_expr(self, o: ComparisonExpr) -> None:
        self.process_node(o)
        super().visit_comparison_expr(o)

    def visit_index_expr(self, o: IndexExpr) -> None:
        self.process_node(o)
        super().visit_index_expr(o)

    def visit_unary_expr(self, o: UnaryExpr) -> None:
        self.process_node(o)
        super().visit_unary_expr(o)

    def process_node(self, node: Expression) -> None:
        if self.all_nodes:
            if self.typemap is not None:
                self.line = node.line
                self.type(self.typemap.get(node))

    def type(self, t: Optional[Type]) -> None:
        if not t:
            # If an expression does not have a type, it is often due to dead code.
            # Don't count these because there can be an unanalyzed value on a line with other
            # analyzed expressions, which overwrite the TYPE_UNANALYZED.
            self.record_line(self.line, TYPE_UNANALYZED)
            return

        if isinstance(t, AnyType) and t.type_of_any == TypeOfAny.special_form:
            # This is not a real Any type, so don't collect stats for it.
            return

        if isinstance(t, AnyType):
            self.log('  !! Any type around line %d' % self.line)
            self.num_any_exprs += 1
            self.record_line(self.line, TYPE_ANY)
        elif ((not self.all_nodes and is_imprecise(t)) or
              (self.all_nodes and is_imprecise2(t))):
            self.log('  !! Imprecise type around line %d' % self.line)
            self.num_imprecise_exprs += 1
            self.record_line(self.line, TYPE_IMPRECISE)
        else:
            self.num_precise_exprs += 1
            self.record_line(self.line, TYPE_PRECISE)

        for typ in collect_all_inner_types(t) + [t]:
            if isinstance(typ, AnyType):
                if typ.type_of_any == TypeOfAny.from_another_any:
                    assert typ.source_any
                    assert typ.source_any.type_of_any != TypeOfAny.from_another_any
                    typ = typ.source_any
                self.type_of_any_counter[typ.type_of_any] += 1
                self.num_any_types += 1
                if self.line in self.any_line_map:
                    self.any_line_map[self.line].append(typ)
                else:
                    self.any_line_map[self.line] = [typ]
            elif isinstance(typ, Instance):
                if typ.args:
                    if any(is_complex(arg) for arg in typ.args):
                        self.num_complex_types += 1
                    else:
                        self.num_generic_types += 1
                else:
                    self.num_simple_types += 1
            elif isinstance(typ, FunctionLike):
                self.num_function_types += 1
            elif isinstance(typ, TupleType):
                if any(is_complex(item) for item in typ.items):
                    self.num_complex_types += 1
                else:
                    self.num_tuple_types += 1
            elif isinstance(typ, TypeVarType):
                self.num_typevar_types += 1

    def log(self, string: str) -> None:
        self.output.append(string)

    def record_line(self, line: int, precision: int) -> None:
        self.line_map[line] = max(precision,
                                  self.line_map.get(line, TYPE_EMPTY))


def dump_type_stats(tree: MypyFile, path: str, inferred: bool = False,
                    typemap: Optional[Dict[Expression, Type]] = None) -> None:
    if is_special_module(path):
        return
    print(path)
    visitor = StatisticsVisitor(inferred, filename=tree.fullname(), typemap=typemap)
    tree.accept(visitor)
    for line in visitor.output:
        print(line)
    print('  ** precision **')
    print('  precise  ', visitor.num_precise_exprs)
    print('  imprecise', visitor.num_imprecise_exprs)
    print('  any      ', visitor.num_any_exprs)
    print('  ** kinds **')
    print('  simple   ', visitor.num_simple_types)
    print('  generic  ', visitor.num_generic_types)
    print('  function ', visitor.num_function_types)
    print('  tuple    ', visitor.num_tuple_types)
    print('  TypeVar  ', visitor.num_typevar_types)
    print('  complex  ', visitor.num_complex_types)
    print('  any      ', visitor.num_any_types)


def is_special_module(path: str) -> bool:
    return os.path.basename(path) in ('abc.pyi', 'typing.pyi', 'builtins.pyi')


def is_imprecise(t: Type) -> bool:
    return t.accept(HasAnyQuery())


class HasAnyQuery(TypeQuery[bool]):
    def __init__(self) -> None:
        super().__init__(any)

    def visit_any(self, t: AnyType) -> bool:
        return True

    def visit_instance(self, t: Instance) -> bool:
        if t.type.fullname() == 'builtins.tuple':
            return True
        else:
            return super().visit_instance(t)


def is_imprecise2(t: Type) -> bool:
    return t.accept(HasAnyQuery2())


class HasAnyQuery2(HasAnyQuery):
    def visit_callable_type(self, t: CallableType) -> bool:
        # We don't want to flag references to functions with some Any
        # argument types (etc.) since they generally don't mean trouble.
        return False


def is_generic(t: Type) -> bool:
    return isinstance(t, Instance) and bool(t.args)


def is_complex(t: Type) -> bool:
    return is_generic(t) or isinstance(t, (FunctionLike, TupleType,
                                           TypeVarType))


def ensure_dir_exists(dir: str) -> None:
    if not os.path.exists(dir):
        os.makedirs(dir)



from typing import Any, Optional

whitespace_re = ...  # type: Any
string_re = ...  # type: Any
integer_re = ...  # type: Any
name_re = ...  # type: Any
float_re = ...  # type: Any
newline_re = ...  # type: Any
TOKEN_ADD = ...  # type: Any
TOKEN_ASSIGN = ...  # type: Any
TOKEN_COLON = ...  # type: Any
TOKEN_COMMA = ...  # type: Any
TOKEN_DIV = ...  # type: Any
TOKEN_DOT = ...  # type: Any
TOKEN_EQ = ...  # type: Any
TOKEN_FLOORDIV = ...  # type: Any
TOKEN_GT = ...  # type: Any
TOKEN_GTEQ = ...  # type: Any
TOKEN_LBRACE = ...  # type: Any
TOKEN_LBRACKET = ...  # type: Any
TOKEN_LPAREN = ...  # type: Any
TOKEN_LT = ...  # type: Any
TOKEN_LTEQ = ...  # type: Any
TOKEN_MOD = ...  # type: Any
TOKEN_MUL = ...  # type: Any
TOKEN_NE = ...  # type: Any
TOKEN_PIPE = ...  # type: Any
TOKEN_POW = ...  # type: Any
TOKEN_RBRACE = ...  # type: Any
TOKEN_RBRACKET = ...  # type: Any
TOKEN_RPAREN = ...  # type: Any
TOKEN_SEMICOLON = ...  # type: Any
TOKEN_SUB = ...  # type: Any
TOKEN_TILDE = ...  # type: Any
TOKEN_WHITESPACE = ...  # type: Any
TOKEN_FLOAT = ...  # type: Any
TOKEN_INTEGER = ...  # type: Any
TOKEN_NAME = ...  # type: Any
TOKEN_STRING = ...  # type: Any
TOKEN_OPERATOR = ...  # type: Any
TOKEN_BLOCK_BEGIN = ...  # type: Any
TOKEN_BLOCK_END = ...  # type: Any
TOKEN_VARIABLE_BEGIN = ...  # type: Any
TOKEN_VARIABLE_END = ...  # type: Any
TOKEN_RAW_BEGIN = ...  # type: Any
TOKEN_RAW_END = ...  # type: Any
TOKEN_COMMENT_BEGIN = ...  # type: Any
TOKEN_COMMENT_END = ...  # type: Any
TOKEN_COMMENT = ...  # type: Any
TOKEN_LINESTATEMENT_BEGIN = ...  # type: Any
TOKEN_LINESTATEMENT_END = ...  # type: Any
TOKEN_LINECOMMENT_BEGIN = ...  # type: Any
TOKEN_LINECOMMENT_END = ...  # type: Any
TOKEN_LINECOMMENT = ...  # type: Any
TOKEN_DATA = ...  # type: Any
TOKEN_INITIAL = ...  # type: Any
TOKEN_EOF = ...  # type: Any
operators = ...  # type: Any
reverse_operators = ...  # type: Any
operator_re = ...  # type: Any
ignored_tokens = ...  # type: Any
ignore_if_empty = ...  # type: Any

def describe_token(token): ...
def describe_token_expr(expr): ...
def count_newlines(value): ...
def compile_rules(environment): ...

class Failure:
    message = ...  # type: Any
    error_class = ...  # type: Any
    def __init__(self, message, cls: Any = ...) -> None: ...
    def __call__(self, lineno, filename): ...

class Token(tuple):
    lineno = ...  # type: Any
    type = ...  # type: Any
    value = ...  # type: Any
    def __new__(cls, lineno, type, value): ...
    def test(self, expr): ...
    def test_any(self, *iterable): ...

class TokenStreamIterator:
    stream = ...  # type: Any
    def __init__(self, stream) -> None: ...
    def __iter__(self): ...
    def __next__(self): ...

class TokenStream:
    name = ...  # type: Any
    filename = ...  # type: Any
    closed = ...  # type: bool
    current = ...  # type: Any
    def __init__(self, generator, name, filename) -> None: ...
    def __iter__(self): ...
    def __bool__(self): ...
    __nonzero__ = ...  # type: Any
    eos = ...  # type: Any
    def push(self, token): ...
    def look(self): ...
    def skip(self, n: int = ...): ...
    def next_if(self, expr): ...
    def skip_if(self, expr): ...
    def __next__(self): ...
    def close(self): ...
    def expect(self, expr): ...

def get_lexer(environment): ...

class Lexer:
    newline_sequence = ...  # type: Any
    keep_trailing_newline = ...  # type: Any
    rules = ...  # type: Any
    def __init__(self, environment) -> None: ...
    def tokenize(self, source, name: Optional[Any] = ..., filename: Optional[Any] = ..., state: Optional[Any] = ...): ...
    def wrap(self, stream, name: Optional[Any] = ..., filename: Optional[Any] = ...): ...
    def tokeniter(self, source, name, filename: Optional[Any] = ..., state: Optional[Any] = ...): ...



from typing import Any
import io

BUFFER_SIZE = ...  # type: Any

class BaseStream(io.BufferedIOBase): ...

class DecompressReader(io.RawIOBase):
    def readable(self): ...
    def __init__(self, fp, decomp_factory, trailing_error=..., **decomp_args): ...
    def close(self): ...
    def seekable(self): ...
    def readinto(self, b): ...
    def read(self, size=-1): ...
    def seek(self, offset, whence=...): ...
    def tell(self): ...



from typing import Container, List, Union, TypeVar, Type, Any, Optional, Tuple
from . import _ValidatorType

_T = TypeVar('_T')

def instance_of(type: Union[Tuple[Type[_T], ...], Type[_T]]) -> _ValidatorType[_T]: ...
def provides(interface: Any) -> _ValidatorType[Any]: ...
def optional(validator: Union[_ValidatorType[_T], List[_ValidatorType[_T]]]) -> _ValidatorType[Optional[_T]]: ...
def in_(options: Container[_T]) -> _ValidatorType[_T]: ...
def and_(*validators: _ValidatorType[_T]) -> _ValidatorType[_T]: ...



# Stubs for re
# Ron Murawski <ron@horizonchess.com>
# 'bytes' support added by Jukka Lehtosalo

# based on: http://docs.python.org/3.2/library/re.html
# and http://hg.python.org/cpython/file/618ea5612e83/Lib/re.py

import sys
from typing import (
    List, Iterator, overload, Callable, Tuple, Sequence, Dict,
    Generic, AnyStr, Match, Pattern, Any, Optional, Union
)

# ----- re variables and constants -----
if sys.version_info >= (3, 6):
    import enum
    class RegexFlag(enum.IntFlag):
        A = 0
        ASCII = 0
        DEBUG = 0
        I = 0
        IGNORECASE = 0
        L = 0
        LOCALE = 0
        M = 0
        MULTILINE = 0
        S = 0
        DOTALL = 0
        X = 0
        VERBOSE = 0
        U = 0
        UNICODE = 0
        T = 0
        TEMPLATE = 0

    A = RegexFlag.A
    ASCII = RegexFlag.ASCII
    DEBUG = RegexFlag.DEBUG
    I = RegexFlag.I
    IGNORECASE = RegexFlag.IGNORECASE
    L = RegexFlag.L
    LOCALE = RegexFlag.LOCALE
    M = RegexFlag.M
    MULTILINE = RegexFlag.MULTILINE
    S = RegexFlag.S
    DOTALL = RegexFlag.DOTALL
    X = RegexFlag.X
    VERBOSE = RegexFlag.VERBOSE
    U = RegexFlag.U
    UNICODE = RegexFlag.UNICODE
    T = RegexFlag.T
    TEMPLATE = RegexFlag.TEMPLATE
    _FlagsType = Union[int, RegexFlag]
else:
    A = 0
    ASCII = 0
    DEBUG = 0
    I = 0
    IGNORECASE = 0
    L = 0
    LOCALE = 0
    M = 0
    MULTILINE = 0
    S = 0
    DOTALL = 0
    X = 0
    VERBOSE = 0
    U = 0
    UNICODE = 0
    T = 0
    TEMPLATE = 0
    _FlagsType = int

class error(Exception): ...

@overload
def compile(pattern: AnyStr, flags: _FlagsType = ...) -> Pattern[AnyStr]: ...
@overload
def compile(pattern: Pattern[AnyStr], flags: _FlagsType = ...) -> Pattern[AnyStr]: ...

@overload
def search(pattern: AnyStr, string: AnyStr, flags: _FlagsType = ...) -> Optional[Match[AnyStr]]: ...
@overload
def search(pattern: Pattern[AnyStr], string: AnyStr, flags: _FlagsType = ...) -> Optional[Match[AnyStr]]: ...

@overload
def match(pattern: AnyStr, string: AnyStr, flags: _FlagsType = ...) -> Optional[Match[AnyStr]]: ...
@overload
def match(pattern: Pattern[AnyStr], string: AnyStr, flags: _FlagsType = ...) -> Optional[Match[AnyStr]]: ...

# New in Python 3.4
@overload
def fullmatch(pattern: AnyStr, string: AnyStr, flags: _FlagsType = ...) -> Optional[Match[AnyStr]]: ...
@overload
def fullmatch(pattern: Pattern[AnyStr], string: AnyStr, flags: _FlagsType = ...) -> Optional[Match[AnyStr]]: ...

@overload
def split(pattern: AnyStr, string: AnyStr,
          maxsplit: int = ..., flags: _FlagsType = ...) -> List[AnyStr]: ...
@overload
def split(pattern: Pattern[AnyStr], string: AnyStr,
          maxsplit: int = ..., flags: _FlagsType = ...) -> List[AnyStr]: ...

@overload
def findall(pattern: AnyStr, string: AnyStr, flags: _FlagsType = ...) -> List[Any]: ...
@overload
def findall(pattern: Pattern[AnyStr], string: AnyStr, flags: _FlagsType = ...) -> List[Any]: ...

# Return an iterator yielding match objects over all non-overlapping matches
# for the RE pattern in string. The string is scanned left-to-right, and
# matches are returned in the order found. Empty matches are included in the
# result unless they touch the beginning of another match.
@overload
def finditer(pattern: AnyStr, string: AnyStr,
             flags: _FlagsType = ...) -> Iterator[Match[AnyStr]]: ...
@overload
def finditer(pattern: Pattern[AnyStr], string: AnyStr,
             flags: _FlagsType = ...) -> Iterator[Match[AnyStr]]: ...

@overload
def sub(pattern: AnyStr, repl: AnyStr, string: AnyStr, count: int = ...,
        flags: _FlagsType = ...) -> AnyStr: ...
@overload
def sub(pattern: AnyStr, repl: Callable[[Match[AnyStr]], AnyStr],
        string: AnyStr, count: int = ..., flags: _FlagsType = ...) -> AnyStr: ...
@overload
def sub(pattern: Pattern[AnyStr], repl: AnyStr, string: AnyStr, count: int = ...,
        flags: _FlagsType = ...) -> AnyStr: ...
@overload
def sub(pattern: Pattern[AnyStr], repl: Callable[[Match[AnyStr]], AnyStr],
        string: AnyStr, count: int = ..., flags: _FlagsType = ...) -> AnyStr: ...

@overload
def subn(pattern: AnyStr, repl: AnyStr, string: AnyStr, count: int = ...,
         flags: _FlagsType = ...) -> Tuple[AnyStr, int]: ...
@overload
def subn(pattern: AnyStr, repl: Callable[[Match[AnyStr]], AnyStr],
         string: AnyStr, count: int = ...,
         flags: _FlagsType = ...) -> Tuple[AnyStr, int]: ...
@overload
def subn(pattern: Pattern[AnyStr], repl: AnyStr, string: AnyStr, count: int = ...,
         flags: _FlagsType = ...) -> Tuple[AnyStr, int]: ...
@overload
def subn(pattern: Pattern[AnyStr], repl: Callable[[Match[AnyStr]], AnyStr],
         string: AnyStr, count: int = ...,
         flags: _FlagsType = ...) -> Tuple[AnyStr, int]: ...

def escape(string: AnyStr) -> AnyStr: ...

def purge() -> None: ...
def template(pattern: Union[AnyStr, Pattern[AnyStr]], flags: _FlagsType = ...) -> Pattern[AnyStr]: ...



# Stub for termcolor: https://pypi.python.org/pypi/termcolor
from typing import Any, Iterable, Optional, Text


def colored(
    text: Text,
    color: Optional[Text] = ...,
    on_color: Optional[Text] = ...,
    attrs: Optional[Iterable[Text]] = ...,
) -> Text:
    ...


def cprint(
    text: Text,
    color: Optional[Text] = ...,
    on_color: Optional[Text] = ...,
    attrs: Optional[Iterable[Text]] = ...,
    **kwargs: Any,
) -> None:
    ...



from typing import Any, Union, Text

__revision__ = ...  # type: str

class ARC4Cipher:
    block_size = ...  # type: int
    key_size = ...  # type: int
    def __init__(self, key: Union[bytes, Text], *args, **kwargs) -> None: ...
    def encrypt(self, plaintext): ...
    def decrypt(self, ciphertext): ...

def new(key: Union[bytes, Text], *args, **kwargs) -> ARC4Cipher: ...

block_size = ...  # type: int
key_size = ...  # type: int



from typing import Any

SERVER_STATUS_IN_TRANS = ...  # type: Any
SERVER_STATUS_AUTOCOMMIT = ...  # type: Any
SERVER_MORE_RESULTS_EXISTS = ...  # type: Any
SERVER_QUERY_NO_GOOD_INDEX_USED = ...  # type: Any
SERVER_QUERY_NO_INDEX_USED = ...  # type: Any
SERVER_STATUS_CURSOR_EXISTS = ...  # type: Any
SERVER_STATUS_LAST_ROW_SENT = ...  # type: Any
SERVER_STATUS_DB_DROPPED = ...  # type: Any
SERVER_STATUS_NO_BACKSLASH_ESCAPES = ...  # type: Any
SERVER_STATUS_METADATA_CHANGED = ...  # type: Any



# Stubs for decimal (Python 3.4)

from typing import (
    Any, Union, SupportsInt, SupportsFloat, SupportsAbs, SupportsRound, Sequence,
    Tuple, NamedTuple, Dict
)

_Decimal = Union[Decimal, int]
_ComparableNum = Union[Decimal, int, float]

BasicContext = ...  # type: Context
DefaultContext = ...  # type: Context
ExtendedContext = ...  # type: Context
HAVE_THREADS = ...  # type: bool
MAX_EMAX = ...  # type: int
MAX_PREC = ...  # type: int
MIN_EMIN = ...  # type: int
MIN_ETINY = ...  # type: int
ROUND_05UP = ...  # type: str
ROUND_CEILING = ...  # type: str
ROUND_DOWN = ...  # type: str
ROUND_FLOOR = ...  # type: str
ROUND_HALF_DOWN = ...  # type: str
ROUND_HALF_EVEN = ...  # type: str
ROUND_HALF_UP = ...  # type: str
ROUND_UP = ...  # type: str

def getcontext() -> Context: ...
def localcontext(ctx: Context = ...) -> _ContextManager: ...
def setcontext(c: Context) -> None: ...

DecimalTuple = NamedTuple('DecimalTuple',
                          [('sign', int),
                           ('digits', Sequence[int]),  # TODO: Use Tuple[int, ...]
                           ('exponent', int)])

class _ContextManager:
    def __enter__(self) -> Context: ...
    def __exit__(self, t, v, tb) -> None: ...

class Context:
    Emax = ...  # type: int
    Emin = ...  # type: int
    capitals = ...  # type: int
    clamp = ...  # type: int
    prec = ...  # type: int
    rounding = ...  # type: str
    traps = ...  # type: Dict[type, bool]
    def __init__(self, prec: int = ..., rounding: str = ..., Emin: int = ..., Emax: int = ...,
                 capitals: int = ..., clamp: int = ..., flags=..., traps=...,
                 _ignored_flags=...) -> None: ...
    def Etiny(self): ...
    def Etop(self): ...
    def abs(self, x: _Decimal) -> Decimal: ...
    def add(self, x: _Decimal, y: _Decimal) -> Decimal: ...
    def canonical(self, x): ...
    def clear_flags(self): ...
    def clear_traps(self): ...
    def compare(self, x, y): ...
    def compare_signal(self, x, y): ...
    def compare_total(self, x, y): ...
    def compare_total_mag(self, x, y): ...
    def copy(self): ...
    def copy_abs(self, x): ...
    def copy_decimal(self, x): ...
    def copy_negate(self, x): ...
    def copy_sign(self, x, y): ...
    def create_decimal(self, x): ...
    def create_decimal_from_float(self, f): ...
    def divide(self, x, y): ...
    def divide_int(self, x, y): ...
    def divmod(self, x, y): ...
    def exp(self, x): ...
    def fma(self, x, y, z): ...
    def is_canonical(self, x): ...
    def is_finite(self, x): ...
    def is_infinite(self, x): ...
    def is_nan(self, x): ...
    def is_normal(self, x): ...
    def is_qnan(self, x): ...
    def is_signed(self, x): ...
    def is_snan(self): ...
    def is_subnormal(self, x): ...
    def is_zero(self, x): ...
    def ln(self, x): ...
    def log10(self, x): ...
    def logb(self, x): ...
    def logical_and(self, x, y): ...
    def logical_invert(self, x): ...
    def logical_or(self, x, y): ...
    def logical_xor(self, x, y): ...
    def max(self, x, y): ...
    def max_mag(self, x, y): ...
    def min(self, x, y): ...
    def min_mag(self, x, y): ...
    def minus(self, x): ...
    def multiply(self, x, y): ...
    def next_minus(self, x): ...
    def next_plus(self, x): ...
    def next_toward(self, x): ...
    def normalize(self, x): ...
    def number_class(self, x): ...
    def plus(self, x): ...
    def power(self, x, y): ...
    def quantize(self, x, y): ...
    def radix(self): ...
    def remainder(self, x, y): ...
    def remainder_near(self, x, y): ...
    def rotate(self, x, y): ...
    def same_quantum(self, x, y): ...
    def scaleb(self, x, y): ...
    def shift(self, x, y): ...
    def sqrt(self, x): ...
    def subtract(self, x, y): ...
    def to_eng_string(self, x): ...
    def to_integral(self, x): ...
    def to_integral_exact(self, x): ...
    def to_integral_value(self, x): ...
    def to_sci_string(self, x): ...
    def __copy__(self) -> Context: ...
    def __delattr__(self, name): ...
    def __reduce__(self): ...

class ConversionSyntax(InvalidOperation): ...

class Decimal(SupportsInt, SupportsFloat, SupportsAbs[Decimal], SupportsRound[int]):
    # TODO: SupportsCeil, SupportsFloor, SupportsTrunc?

    def __init__(cls, value: Union[_Decimal, float, str,
                                   Tuple[int, Sequence[int], int]] = ...,
                 context: Context = ...) -> None: ...

    @property
    def imag(self) -> Decimal: ...
    @property
    def real(self) -> Decimal: ...

    def adjusted(self) -> int: ...
    def as_tuple(self) -> DecimalTuple: ...
    def canonical(self) -> Decimal: ...
    def compare(self, other: _Decimal, context: Context = ...) -> Decimal: ...
    def compare_signal(self, other: _Decimal, context: Context = ...) -> Decimal: ...
    def compare_total(self, other: _Decimal, context: Context = ...) -> Decimal: ...
    def compare_total_mag(self, other: _Decimal, context: Context = ...) -> Decimal: ...
    def conjugate(self) -> Decimal: ...
    def copy_abs(self) -> Decimal: ...
    def copy_negate(self) -> Decimal: ...
    def copy_sign(self, other: _Decimal, context: Context = ...) -> Decimal: ...
    def exp(self, context: Context = ...) -> Decimal: ...
    def fma(self, other: _Decimal, third: _Decimal, context: Context = ...) -> Decimal: ...
    @classmethod
    def from_float(cls, f: float) -> Decimal: ...
    def is_canonical(self) -> bool: ...
    def is_finite(self) -> bool: ...
    def is_infinite(self) -> bool: ...
    def is_nan(self) -> bool: ...
    def is_normal(self, context: Context = ...) -> bool: ...
    def is_qnan(self) -> bool: ...
    def is_signed(self) -> bool: ...
    def is_snan(self) -> bool: ...
    def is_subnormal(self, context: Context = ...) -> bool: ...
    def is_zero(self) -> bool: ...
    def ln(self, context: Context = ...) -> Decimal: ...
    def log10(self, context: Context = ...) -> Decimal: ...
    def logb(self, context: Context = ...) -> Decimal: ...
    def logical_and(self, other: _Decimal, context: Context = ...) -> Decimal: ...
    def logical_invert(self, context: Context = ...) -> Decimal: ...
    def logical_or(self, other: _Decimal, context: Context = ...) -> Decimal: ...
    def logical_xor(self, other: _Decimal, context: Context = ...) -> Decimal: ...
    def max(self, other: _Decimal, context: Context = ...) -> Decimal: ...
    def max_mag(self, other: _Decimal, context: Context = ...) -> Decimal: ...
    def min(self, other: _Decimal, context: Context = ...) -> Decimal: ...
    def min_mag(self, other: _Decimal, context: Context = ...) -> Decimal: ...
    def next_minus(self, context: Context = ...) -> Decimal: ...
    def next_plus(self, context: Context = ...) -> Decimal: ...
    def next_toward(self, other: _Decimal, context: Context = ...) -> Decimal: ...
    def normalize(self, context: Context = ...) -> Decimal: ...
    def number_class(self, context: Context = ...) -> str: ...
    def quantize(self, exp: _Decimal, rounding: str = ...,
                 context: Context = ...) -> Decimal: ...
    def radix(self) -> Decimal: ...
    def remainder_near(self, other: _Decimal, context: Context = ...) -> Decimal: ...
    def rotate(self, other: _Decimal, context: Context = ...) -> Decimal: ...
    def same_quantum(self, other: _Decimal, context: Context = ...) -> bool: ...
    def scaleb(self, other: _Decimal, context: Context = ...) -> Decimal: ...
    def shift(self, other: _Decimal, context: Context = ...) -> Decimal: ...
    def sqrt(self, context: Context = ...) -> Decimal: ...
    def to_eng_string(self, context: Context = ...) -> str: ...
    def to_integral(self, rounding: str = ..., context: Context = ...) -> Decimal: ...
    def to_integral_exact(self, rounding: str = ..., context: Context = ...) -> Decimal: ...
    def to_integral_value(self, rounding: str = ..., context: Context = ...) -> Decimal: ...
    def __abs__(self) -> Decimal: ...
    def __add__(self, other: _Decimal) -> Decimal: ...
    def __bool__(self) -> bool: ...
    def __ceil__(self) -> int: ...
    def __complex__(self) -> complex: ...
    def __copy__(self) -> Decimal: ...
    def __deepcopy__(self) -> Decimal: ...
    def __divmod__(self, other: _Decimal) -> Tuple[Decimal, Decimal]: ...
    def __eq__(self, other: object) -> bool: ...
    def __float__(self) -> float: ...
    def __floor__(self) -> int: ...
    def __floordiv__(self, other: _Decimal) -> Decimal: ...
    def __format__(self, specifier, context=..., _localeconv=...) -> str: ...
    def __ge__(self, other: _ComparableNum) -> bool: ...
    def __gt__(self, other: _ComparableNum) -> bool: ...
    def __hash__(self) -> int: ...
    def __int__(self) -> int: ...
    def __le__(self, other: _ComparableNum) -> bool: ...
    def __lt__(self, other: _ComparableNum) -> bool: ...
    def __mod__(self, other: _Decimal) -> Decimal: ...
    def __mul__(self, other: _Decimal) -> Decimal: ...
    def __ne__(self, other: object) -> bool: ...
    def __neg__(self) -> Decimal: ...
    def __pos__(self) -> Decimal: ...
    def __pow__(self, other: _Decimal) -> Decimal: ...
    def __radd__(self, other: int) -> Decimal: ...
    def __rdivmod__(self, other: int) -> Tuple[Decimal, Decimal]: ...
    def __reduce__(self): ...
    def __rfloordiv__(self, other: int) -> Decimal: ...
    def __rmod__(self, other: int) -> Decimal: ...
    def __rmul__(self, other: int) -> Decimal: ...
    def __round__(self, n=...) -> int: ...
    def __rpow__(self, other: int) -> Decimal: ...
    def __rsub__(self, other: int) -> Decimal: ...
    def __rtruediv__(self, other: int) -> Decimal: ...
    def __sub__(self, other: _Decimal) -> Decimal: ...
    def __truediv__(self, other: _Decimal) -> Decimal: ...
    def __trunc__(self) -> int: ...

class DecimalException(ArithmeticError): ...

class Clamped(DecimalException): ...

class DivisionByZero(DecimalException, ZeroDivisionError): ...

class DivisionImpossible(InvalidOperation): ...

class DivisionUndefined(InvalidOperation, ZeroDivisionError): ...

class FloatOperation(DecimalException, TypeError): ...

class Inexact(DecimalException): ...

class InvalidContext(InvalidOperation): ...

class InvalidOperation(DecimalException): ...

class Overflow(Inexact, Rounded): ...

class Rounded(DecimalException): ...

class Subnormal(DecimalException): ...

class Underflow(Inexact, Rounded, Subnormal): ...



# Filip Hron <filip.hron@gmail.com>
# based heavily on Andrey Vlasovskikh's python-skeletons https://github.com/JetBrains/python-skeletons/blob/master/sqlite3.py

import sys
from typing import Any, Callable, Iterable, Iterator, List, Optional, Text, Tuple, Type, TypeVar, Union
from datetime import date, time, datetime

_T = TypeVar('_T')

paramstyle: str
threadsafety: int
apilevel: str
Date = date
Time = time
Timestamp = datetime

def DateFromTicks(ticks): ...
def TimeFromTicks(ticks): ...
def TimestampFromTicks(ticks): ...

version_info: str
sqlite_version_info: Tuple[int, int, int]
if sys.version_info >= (3,):
    Binary = memoryview
else:
    Binary = buffer

def register_adapters_and_converters(): ...

# The remaining definitions are imported from _sqlite3.

PARSE_COLNAMES = ...  # type: int
PARSE_DECLTYPES = ...  # type: int
SQLITE_ALTER_TABLE = ...  # type: int
SQLITE_ANALYZE = ...  # type: int
SQLITE_ATTACH = ...  # type: int
SQLITE_CREATE_INDEX = ...  # type: int
SQLITE_CREATE_TABLE = ...  # type: int
SQLITE_CREATE_TEMP_INDEX = ...  # type: int
SQLITE_CREATE_TEMP_TABLE = ...  # type: int
SQLITE_CREATE_TEMP_TRIGGER = ...  # type: int
SQLITE_CREATE_TEMP_VIEW = ...  # type: int
SQLITE_CREATE_TRIGGER = ...  # type: int
SQLITE_CREATE_VIEW = ...  # type: int
SQLITE_DELETE = ...  # type: int
SQLITE_DENY = ...  # type: int
SQLITE_DETACH = ...  # type: int
SQLITE_DROP_INDEX = ...  # type: int
SQLITE_DROP_TABLE = ...  # type: int
SQLITE_DROP_TEMP_INDEX = ...  # type: int
SQLITE_DROP_TEMP_TABLE = ...  # type: int
SQLITE_DROP_TEMP_TRIGGER = ...  # type: int
SQLITE_DROP_TEMP_VIEW = ...  # type: int
SQLITE_DROP_TRIGGER = ...  # type: int
SQLITE_DROP_VIEW = ...  # type: int
SQLITE_IGNORE = ...  # type: int
SQLITE_INSERT = ...  # type: int
SQLITE_OK = ...  # type: int
SQLITE_PRAGMA = ...  # type: int
SQLITE_READ = ...  # type: int
SQLITE_REINDEX = ...  # type: int
SQLITE_SELECT = ...  # type: int
SQLITE_TRANSACTION = ...  # type: int
SQLITE_UPDATE = ...  # type: int
adapters = ...  # type: Any
converters = ...  # type: Any
sqlite_version = ...  # type: str
version = ...  # type: str

# TODO: adapt needs to get probed
def adapt(obj, protocol, alternate): ...
def complete_statement(sql: str) -> bool: ...
if sys.version_info >= (3, 4):
    def connect(database: Union[bytes, Text],
                timeout: float = ...,
                detect_types: int = ...,
                isolation_level: Optional[str] = ...,
                check_same_thread: bool = ...,
                factory: Optional[Type[Connection]] = ...,
                cached_statements: int = ...,
                uri: bool = ...) -> Connection: ...
else:
    def connect(database: Union[bytes, Text],
                timeout: float = ...,
                detect_types: int = ...,
                isolation_level: Optional[str] = ...,
                check_same_thread: bool = ...,
                factory: Optional[Type[Connection]] = ...,
                cached_statements: int = ...) -> Connection: ...
def enable_callback_tracebacks(flag: bool) -> None: ...
def enable_shared_cache(do_enable: int) -> None: ...
def register_adapter(type: Type[_T], callable: Callable[[_T], Union[int, float, str, bytes]]) -> None: ...
def register_converter(typename: str, callable: Callable[[bytes], Any]) -> None: ...

class Cache(object):
    def __init__(self, *args, **kwargs) -> None: ...
    def display(self, *args, **kwargs) -> None: ...
    def get(self, *args, **kwargs) -> None: ...

class Connection(object):
    DataError = ...  # type: Any
    DatabaseError = ...  # type: Any
    Error = ...  # type: Any
    IntegrityError = ...  # type: Any
    InterfaceError = ...  # type: Any
    InternalError = ...  # type: Any
    NotSupportedError = ...  # type: Any
    OperationalError = ...  # type: Any
    ProgrammingError = ...  # type: Any
    Warning = ...  # type: Any
    in_transaction = ...  # type: Any
    isolation_level = ...  # type: Any
    row_factory = ...  # type: Any
    text_factory = ...  # type: Any
    total_changes = ...  # type: Any
    def __init__(self, *args, **kwargs): ...
    def close(self) -> None: ...
    def commit(self) -> None: ...
    def create_aggregate(self, name: str, num_params: int, aggregate_class: type) -> None: ...
    def create_collation(self, name: str, callable: Any) -> None: ...
    def create_function(self, name: str, num_params: int, func: Any) -> None: ...
    def cursor(self, cursorClass: Optional[type] = ...) -> Cursor: ...
    def execute(self, sql: str, parameters: Iterable = ...) -> Cursor: ...
    # TODO: please check in executemany() if seq_of_parameters type is possible like this
    def executemany(self, sql: str, seq_of_parameters: Iterable[Iterable]) -> Cursor: ...
    def executescript(self, sql_script: Union[bytes, Text]) -> Cursor: ...
    def interrupt(self, *args, **kwargs) -> None: ...
    def iterdump(self, *args, **kwargs) -> None: ...
    def rollback(self, *args, **kwargs) -> None: ...
    # TODO: set_authorizer(authorzer_callback)
    # see https://docs.python.org/2/library/sqlite3.html#sqlite3.Connection.set_authorizer
    # returns [SQLITE_OK, SQLITE_DENY, SQLITE_IGNORE] so perhaps int
    def set_authorizer(self, *args, **kwargs) -> None: ...
    # set_progress_handler(handler, n) -> see https://docs.python.org/2/library/sqlite3.html#sqlite3.Connection.set_progress_handler
    def set_progress_handler(self, *args, **kwargs) -> None: ...
    def set_trace_callback(self, *args, **kwargs): ...
    if sys.version_info >= (3, 7):
        def backup(self, target: Connection, *, pages: int = ...,
                   progress: Optional[Callable[[int, int, int], object]] = ..., name: str = ...,
                   sleep: float = ...) -> None: ...
    def __call__(self, *args, **kwargs): ...
    def __enter__(self, *args, **kwargs): ...
    def __exit__(self, *args, **kwargs): ...

class Cursor(Iterator[Any]):
    arraysize = ...  # type: Any
    connection = ...  # type: Any
    description = ...  # type: Any
    lastrowid = ...  # type: Any
    row_factory = ...  # type: Any
    rowcount = ...  # type: Any
    # TODO: Cursor class accepts exactly 1 argument
    # required type is sqlite3.Connection (which is imported as _Connection)
    # however, the name of the __init__ variable is unknown
    def __init__(self, *args, **kwargs): ...
    def close(self, *args, **kwargs): ...
    def execute(self, sql: str, parameters: Iterable = ...) -> Cursor: ...
    def executemany(self, sql: str, seq_of_parameters: Iterable[Iterable]): ...
    def executescript(self, sql_script: Union[bytes, Text]) -> Cursor: ...
    def fetchall(self) -> List[Any]: ...
    def fetchmany(self, size: Optional[int] = ...) -> List[Any]: ...
    def fetchone(self) -> Any: ...
    def setinputsizes(self, *args, **kwargs): ...
    def setoutputsize(self, *args, **kwargs): ...
    def __iter__(self) -> Cursor: ...
    if sys.version_info >= (3, 0):
        def __next__(self) -> Any: ...
    else:
        def next(self) -> Any: ...


class DataError(DatabaseError): ...

class DatabaseError(Error): ...

class Error(Exception): ...

class IntegrityError(DatabaseError): ...

class InterfaceError(Error): ...

class InternalError(DatabaseError): ...

class NotSupportedError(DatabaseError): ...

class OperationalError(DatabaseError): ...

class OptimizedUnicode(object):
    maketrans = ...  # type: Any
    def __init__(self, *args, **kwargs): ...
    def capitalize(self, *args, **kwargs): ...
    def casefold(self, *args, **kwargs): ...
    def center(self, *args, **kwargs): ...
    def count(self, *args, **kwargs): ...
    def encode(self, *args, **kwargs): ...
    def endswith(self, *args, **kwargs): ...
    def expandtabs(self, *args, **kwargs): ...
    def find(self, *args, **kwargs): ...
    def format(self, *args, **kwargs): ...
    def format_map(self, *args, **kwargs): ...
    def index(self, *args, **kwargs): ...
    def isalnum(self, *args, **kwargs): ...
    def isalpha(self, *args, **kwargs): ...
    def isdecimal(self, *args, **kwargs): ...
    def isdigit(self, *args, **kwargs): ...
    def isidentifier(self, *args, **kwargs): ...
    def islower(self, *args, **kwargs): ...
    def isnumeric(self, *args, **kwargs): ...
    def isprintable(self, *args, **kwargs): ...
    def isspace(self, *args, **kwargs): ...
    def istitle(self, *args, **kwargs): ...
    def isupper(self, *args, **kwargs): ...
    def join(self, *args, **kwargs): ...
    def ljust(self, *args, **kwargs): ...
    def lower(self, *args, **kwargs): ...
    def lstrip(self, *args, **kwargs): ...
    def partition(self, *args, **kwargs): ...
    def replace(self, *args, **kwargs): ...
    def rfind(self, *args, **kwargs): ...
    def rindex(self, *args, **kwargs): ...
    def rjust(self, *args, **kwargs): ...
    def rpartition(self, *args, **kwargs): ...
    def rsplit(self, *args, **kwargs): ...
    def rstrip(self, *args, **kwargs): ...
    def split(self, *args, **kwargs): ...
    def splitlines(self, *args, **kwargs): ...
    def startswith(self, *args, **kwargs): ...
    def strip(self, *args, **kwargs): ...
    def swapcase(self, *args, **kwargs): ...
    def title(self, *args, **kwargs): ...
    def translate(self, *args, **kwargs): ...
    def upper(self, *args, **kwargs): ...
    def zfill(self, *args, **kwargs): ...
    def __add__(self, other): ...
    def __contains__(self, *args, **kwargs): ...
    def __eq__(self, other): ...
    def __format__(self, *args, **kwargs): ...
    def __ge__(self, other): ...
    def __getitem__(self, index): ...
    def __getnewargs__(self, *args, **kwargs): ...
    def __gt__(self, other): ...
    def __hash__(self): ...
    def __iter__(self): ...
    def __le__(self, other): ...
    def __len__(self, *args, **kwargs): ...
    def __lt__(self, other): ...
    def __mod__(self, other): ...
    def __mul__(self, other): ...
    def __ne__(self, other): ...
    def __rmod__(self, other): ...
    def __rmul__(self, other): ...

class PrepareProtocol(object):
    def __init__(self, *args, **kwargs): ...

class ProgrammingError(DatabaseError): ...

class Row(object):
    def __init__(self, *args, **kwargs): ...
    def keys(self, *args, **kwargs): ...
    def __eq__(self, other): ...
    def __ge__(self, other): ...
    def __getitem__(self, index): ...
    def __gt__(self, other): ...
    def __hash__(self): ...
    def __iter__(self): ...
    def __le__(self, other): ...
    def __len__(self, *args, **kwargs): ...
    def __lt__(self, other): ...
    def __ne__(self, other): ...

class Statement(object):
    def __init__(self, *args, **kwargs): ...

class Warning(Exception): ...



from typing import Generator

def walk(self) -> Generator: ...
def body_line_iterator(msg, decode: bool = ...) -> Generator: ...
def typed_subpart_iterator(msg, maintype=..., subtype=...) -> Generator: ...



"""Stub file for the '_stat' module."""

SF_APPEND = ...  # type: int
SF_ARCHIVED = ...  # type: int
SF_IMMUTABLE = ...  # type: int
SF_NOUNLINK = ...  # type: int
SF_SNAPSHOT = ...  # type: int
ST_ATIME = ...  # type: int
ST_CTIME = ...  # type: int
ST_DEV = ...  # type: int
ST_GID = ...  # type: int
ST_INO = ...  # type: int
ST_MODE = ...  # type: int
ST_MTIME = ...  # type: int
ST_NLINK = ...  # type: int
ST_SIZE = ...  # type: int
ST_UID = ...  # type: int
S_ENFMT = ...  # type: int
S_IEXEC = ...  # type: int
S_IFBLK = ...  # type: int
S_IFCHR = ...  # type: int
S_IFDIR = ...  # type: int
S_IFDOOR = ...  # type: int
S_IFIFO = ...  # type: int
S_IFLNK = ...  # type: int
S_IFPORT = ...  # type: int
S_IFREG = ...  # type: int
S_IFSOCK = ...  # type: int
S_IFWHT = ...  # type: int
S_IREAD = ...  # type: int
S_IRGRP = ...  # type: int
S_IROTH = ...  # type: int
S_IRUSR = ...  # type: int
S_IRWXG = ...  # type: int
S_IRWXO = ...  # type: int
S_IRWXU = ...  # type: int
S_ISGID = ...  # type: int
S_ISUID = ...  # type: int
S_ISVTX = ...  # type: int
S_IWGRP = ...  # type: int
S_IWOTH = ...  # type: int
S_IWRITE = ...  # type: int
S_IWUSR = ...  # type: int
S_IXGRP = ...  # type: int
S_IXOTH = ...  # type: int
S_IXUSR = ...  # type: int
UF_APPEND = ...  # type: int
UF_COMPRESSED = ...  # type: int
UF_HIDDEN = ...  # type: int
UF_IMMUTABLE = ...  # type: int
UF_NODUMP = ...  # type: int
UF_NOUNLINK = ...  # type: int
UF_OPAQUE = ...  # type: int

def S_IMODE(mode: int) -> int: ...
def S_IFMT(mode: int) -> int: ...

def S_ISBLK(mode: int) -> bool: ...
def S_ISCHR(mode: int) -> bool: ...
def S_ISDIR(mode: int) -> bool: ...
def S_ISDOOR(mode: int) -> bool: ...
def S_ISFIFO(mode: int) -> bool: ...
def S_ISLNK(mode: int) -> bool: ...
def S_ISPORT(mode: int) -> bool: ...
def S_ISREG(mode: int) -> bool: ...
def S_ISSOCK(mode: int) -> bool: ...
def S_ISWHT(mode: int) -> bool: ...

def filemode(mode: int) -> str: ...



from google.protobuf.internal.containers import (
    RepeatedCompositeFieldContainer,
)
from google.protobuf.message import (
    Message,
)
from google.protobuf.unittest_no_arena_import_pb2 import (
    ImportNoArenaNestedMessage,
)
from typing import (
    Iterable,
    Optional,
)


class NestedMessage(Message):
    d = ...  # type: int

    def __init__(self,
                 d: Optional[int] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> NestedMessage: ...


class ArenaMessage(Message):

    @property
    def repeated_nested_message(
        self) -> RepeatedCompositeFieldContainer[NestedMessage]: ...

    @property
    def repeated_import_no_arena_message(
        self) -> RepeatedCompositeFieldContainer[ImportNoArenaNestedMessage]: ...

    def __init__(self,
                 repeated_nested_message: Optional[Iterable[NestedMessage]] = ...,
                 repeated_import_no_arena_message: Optional[Iterable[ImportNoArenaNestedMessage]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> ArenaMessage: ...



# NB: SocketServer.pyi and socketserver.pyi must remain consistent!
# Stubs for socketserver

from typing import Any, BinaryIO, Optional, Tuple, Type
from socket import SocketType
import sys
import types

class BaseServer:
    address_family = ...  # type: int
    RequestHandlerClass = ...  # type: type
    server_address = ...  # type: Tuple[str, int]
    socket = ...  # type: SocketType
    allow_reuse_address = ...  # type: bool
    request_queue_size = ...  # type: int
    socket_type = ...  # type: int
    timeout = ...  # type: Optional[float]
    def __init__(self, server_address: Tuple[str, int],
                 RequestHandlerClass: type) -> None: ...
    def fileno(self) -> int: ...
    def handle_request(self) -> None: ...
    def serve_forever(self, poll_interval: float = ...) -> None: ...
    def shutdown(self) -> None: ...
    def server_close(self) -> None: ...
    def finish_request(self, request: bytes,
                       client_address: Tuple[str, int]) -> None: ...
    def get_request(self) -> None: ...
    def handle_error(self, request: bytes,
                     client_address: Tuple[str, int]) -> None: ...
    def handle_timeout(self) -> None: ...
    def process_request(self, request: bytes,
                        client_address: Tuple[str, int]) -> None: ...
    def server_activate(self) -> None: ...
    def server_bind(self) -> None: ...
    def verify_request(self, request: bytes,
                       client_address: Tuple[str, int]) -> bool: ...
    if sys.version_info >= (3, 6):
        def __enter__(self) -> BaseServer: ...
        def __exit__(self, exc_type: Optional[Type[BaseException]],
                     exc_val: Optional[BaseException],
                     exc_tb: Optional[types.TracebackType]) -> bool: ...
    if sys.version_info >= (3, 3):
        def service_actions(self) -> None: ...

class TCPServer(BaseServer):
    def __init__(self, server_address: Tuple[str, int],
                 RequestHandlerClass: type,
                 bind_and_activate: bool = ...) -> None: ...

class UDPServer(BaseServer):
    def __init__(self, server_address: Tuple[str, int],
                 RequestHandlerClass: type,
                 bind_and_activate: bool = ...) -> None: ...

if sys.platform != 'win32':
    class UnixStreamServer(BaseServer):
        def __init__(self, server_address: Tuple[str, int],
                     RequestHandlerClass: type,
                     bind_and_activate: bool = ...) -> None: ...

    class UnixDatagramServer(BaseServer):
        def __init__(self, server_address: Tuple[str, int],
                     RequestHandlerClass: type,
                     bind_and_activate: bool = ...) -> None: ...

class ForkingMixIn: ...
class ThreadingMixIn: ...

class ForkingTCPServer(ForkingMixIn, TCPServer): ...
class ForkingUDPServer(ForkingMixIn, UDPServer): ...
class ThreadingTCPServer(ThreadingMixIn, TCPServer): ...
class ThreadingUDPServer(ThreadingMixIn, UDPServer): ...
if sys.platform != 'win32':
    class ThreadingUnixStreamServer(ThreadingMixIn, UnixStreamServer): ...
    class ThreadingUnixDatagramServer(ThreadingMixIn, UnixDatagramServer): ...


class BaseRequestHandler:
    # Those are technically of types, respectively:
    # * Union[SocketType, Tuple[bytes, SocketType]]
    # * Union[Tuple[str, int], str]
    # But there are some concerns that having unions here would cause
    # too much inconvenience to people using it (see
    # https://github.com/python/typeshed/pull/384#issuecomment-234649696)
    request = ...  # type: Any
    client_address = ...  # type: Any

    server = ...  # type: BaseServer
    def setup(self) -> None: ...
    def handle(self) -> None: ...
    def finish(self) -> None: ...

class StreamRequestHandler(BaseRequestHandler):
    rfile = ...  # type: BinaryIO
    wfile = ...  # type: BinaryIO

class DatagramRequestHandler(BaseRequestHandler):
    rfile = ...  # type: BinaryIO
    wfile = ...  # type: BinaryIO



from typing import Any
from jinja2.visitor import NodeTransformer

def optimize(node, environment): ...

class Optimizer(NodeTransformer):
    environment = ...  # type: Any
    def __init__(self, environment) -> None: ...
    def visit_If(self, node): ...
    def fold(self, node): ...
    visit_Add = ...  # type: Any
    visit_Sub = ...  # type: Any
    visit_Mul = ...  # type: Any
    visit_Div = ...  # type: Any
    visit_FloorDiv = ...  # type: Any
    visit_Pow = ...  # type: Any
    visit_Mod = ...  # type: Any
    visit_And = ...  # type: Any
    visit_Or = ...  # type: Any
    visit_Pos = ...  # type: Any
    visit_Neg = ...  # type: Any
    visit_Not = ...  # type: Any
    visit_Compare = ...  # type: Any
    visit_Getitem = ...  # type: Any
    visit_Getattr = ...  # type: Any
    visit_Call = ...  # type: Any
    visit_Filter = ...  # type: Any
    visit_Test = ...  # type: Any
    visit_CondExpr = ...  # type: Any



"""Plugin system for extending mypy."""

from abc import abstractmethod
from functools import partial
from typing import Callable, List, Tuple, Optional, NamedTuple, TypeVar, Dict

from mypy.nodes import (
    Expression, StrExpr, IntExpr, UnaryExpr, Context, DictExpr, ClassDef,
    TypeInfo, SymbolTableNode, MypyFile
)
from mypy.tvar_scope import TypeVarScope
from mypy.types import (
    Type, Instance, CallableType, TypedDictType, UnionType, NoneTyp, TypeVarType,
    AnyType, TypeList, UnboundType, TypeOfAny
)
from mypy.messages import MessageBuilder
from mypy.options import Options


class TypeAnalyzerPluginInterface:
    """Interface for accessing semantic analyzer functionality in plugins."""

    @abstractmethod
    def fail(self, msg: str, ctx: Context) -> None:
        raise NotImplementedError

    @abstractmethod
    def named_type(self, name: str, args: List[Type]) -> Instance:
        raise NotImplementedError

    @abstractmethod
    def analyze_type(self, typ: Type) -> Type:
        raise NotImplementedError

    @abstractmethod
    def analyze_callable_args(self, arglist: TypeList) -> Optional[Tuple[List[Type],
                                                                         List[int],
                                                                         List[Optional[str]]]]:
        raise NotImplementedError


# A context for a hook that semantically analyzes an unbound type.
AnalyzeTypeContext = NamedTuple(
    'AnalyzeTypeContext', [
        ('type', UnboundType),  # Type to analyze
        ('context', Context),
        ('api', TypeAnalyzerPluginInterface)])


class CheckerPluginInterface:
    """Interface for accessing type checker functionality in plugins."""

    msg = None  # type: MessageBuilder

    @abstractmethod
    def named_generic_type(self, name: str, args: List[Type]) -> Instance:
        raise NotImplementedError


class SemanticAnalyzerPluginInterface:
    """Interface for accessing semantic analyzer functionality in plugins."""

    modules = None  # type: Dict[str, MypyFile]
    options = None  # type: Options
    msg = None  # type: MessageBuilder

    @abstractmethod
    def named_type(self, qualified_name: str, args: Optional[List[Type]] = None) -> Instance:
        raise NotImplementedError

    @abstractmethod
    def parse_bool(self, expr: Expression) -> Optional[bool]:
        raise NotImplementedError

    @abstractmethod
    def fail(self, msg: str, ctx: Context, serious: bool = False, *,
             blocker: bool = False) -> None:
        raise NotImplementedError

    @abstractmethod
    def anal_type(self, t: Type, *,
                  tvar_scope: Optional[TypeVarScope] = None,
                  allow_tuple_literal: bool = False,
                  allow_unbound_tvars: bool = False,
                  third_pass: bool = False) -> Type:
        raise NotImplementedError

    @abstractmethod
    def class_type(self, info: TypeInfo) -> Type:
        raise NotImplementedError

    @abstractmethod
    def builtin_type(self, fully_qualified_name: str) -> Instance:
        raise NotImplementedError

    @abstractmethod
    def lookup_fully_qualified(self, name: str) -> SymbolTableNode:
        raise NotImplementedError

    @abstractmethod
    def lookup_fully_qualified_or_none(self, name: str) -> Optional[SymbolTableNode]:
        raise NotImplementedError

    @abstractmethod
    def lookup_qualified(self, name: str, ctx: Context,
                         suppress_errors: bool = False) -> Optional[SymbolTableNode]:
        raise NotImplementedError


# A context for a function hook that infers the return type of a function with
# a special signature.
#
# A no-op callback would just return the inferred return type, but a useful
# callback at least sometimes can infer a more precise type.
FunctionContext = NamedTuple(
    'FunctionContext', [
        ('arg_types', List[List[Type]]),   # List of actual caller types for each formal argument
        ('default_return_type', Type),     # Return type inferred from signature
        ('args', List[List[Expression]]),  # Actual expressions for each formal argument
        ('context', Context),
        ('api', CheckerPluginInterface)])

# A context for a method signature hook that infers a better signature for a
# method.  Note that argument types aren't available yet.  If you need them,
# you have to use a method hook instead.
MethodSigContext = NamedTuple(
    'MethodSigContext', [
        ('type', Type),                       # Base object type for method call
        ('args', List[List[Expression]]),     # Actual expressions for each formal argument
        ('default_signature', CallableType),  # Original signature of the method
        ('context', Context),
        ('api', CheckerPluginInterface)])

# A context for a method hook that infers the return type of a method with a
# special signature.
#
# This is very similar to FunctionContext (only differences are documented).
MethodContext = NamedTuple(
    'MethodContext', [
        ('type', Type),                    # Base object type for method call
        ('arg_types', List[List[Type]]),
        ('default_return_type', Type),
        ('args', List[List[Expression]]),
        ('context', Context),
        ('api', CheckerPluginInterface)])

# A context for an attribute type hook that infers the type of an attribute.
AttributeContext = NamedTuple(
    'AttributeContext', [
        ('type', Type),                # Type of object with attribute
        ('default_attr_type', Type),  # Original attribute type
        ('context', Context),
        ('api', CheckerPluginInterface)])

# A context for a class hook that modifies the class definition.
ClassDefContext = NamedTuple(
    'ClassDecoratorContext', [
        ('cls', ClassDef),       # The class definition
        ('reason', Expression),  # The expression being applied (decorator, metaclass, base class)
        ('api', SemanticAnalyzerPluginInterface)
    ])


class Plugin:
    """Base class of all type checker plugins.

    This defines a no-op plugin.  Subclasses can override some methods to
    provide some actual functionality.

    All get_ methods are treated as pure functions (you should assume that
    results might be cached).

    Look at the comments of various *Context objects for descriptions of
    various hooks.
    """

    def __init__(self, options: Options) -> None:
        self.options = options
        self.python_version = options.python_version

    def get_type_analyze_hook(self, fullname: str
                              ) -> Optional[Callable[[AnalyzeTypeContext], Type]]:
        return None

    def get_function_hook(self, fullname: str
                          ) -> Optional[Callable[[FunctionContext], Type]]:
        return None

    def get_method_signature_hook(self, fullname: str
                                  ) -> Optional[Callable[[MethodSigContext], CallableType]]:
        return None

    def get_method_hook(self, fullname: str
                        ) -> Optional[Callable[[MethodContext], Type]]:
        return None

    def get_attribute_hook(self, fullname: str
                           ) -> Optional[Callable[[AttributeContext], Type]]:
        return None

    def get_class_decorator_hook(self, fullname: str
                                 ) -> Optional[Callable[[ClassDefContext], None]]:
        return None

    def get_metaclass_hook(self, fullname: str
                           ) -> Optional[Callable[[ClassDefContext], None]]:
        return None

    def get_base_class_hook(self, fullname: str
                            ) -> Optional[Callable[[ClassDefContext], None]]:
        return None


T = TypeVar('T')


class ChainedPlugin(Plugin):
    """A plugin that represents a sequence of chained plugins.

    Each lookup method returns the hook for the first plugin that
    reports a match.

    This class should not be subclassed -- use Plugin as the base class
    for all plugins.
    """

    # TODO: Support caching of lookup results (through a LRU cache, for example).

    def __init__(self, options: Options, plugins: List[Plugin]) -> None:
        """Initialize chained plugin.

        Assume that the child plugins aren't mutated (results may be cached).
        """
        super().__init__(options)
        self._plugins = plugins

    def get_type_analyze_hook(self, fullname: str
                              ) -> Optional[Callable[[AnalyzeTypeContext], Type]]:
        return self._find_hook(lambda plugin: plugin.get_type_analyze_hook(fullname))

    def get_function_hook(self, fullname: str
                          ) -> Optional[Callable[[FunctionContext], Type]]:
        return self._find_hook(lambda plugin: plugin.get_function_hook(fullname))

    def get_method_signature_hook(self, fullname: str
                                  ) -> Optional[Callable[[MethodSigContext], CallableType]]:
        return self._find_hook(lambda plugin: plugin.get_method_signature_hook(fullname))

    def get_method_hook(self, fullname: str
                        ) -> Optional[Callable[[MethodContext], Type]]:
        return self._find_hook(lambda plugin: plugin.get_method_hook(fullname))

    def get_attribute_hook(self, fullname: str
                           ) -> Optional[Callable[[AttributeContext], Type]]:
        return self._find_hook(lambda plugin: plugin.get_attribute_hook(fullname))

    def get_class_decorator_hook(self, fullname: str
                                 ) -> Optional[Callable[[ClassDefContext], None]]:
        return self._find_hook(lambda plugin: plugin.get_class_decorator_hook(fullname))

    def get_metaclass_hook(self, fullname: str
                           ) -> Optional[Callable[[ClassDefContext], None]]:
        return self._find_hook(lambda plugin: plugin.get_metaclass_hook(fullname))

    def get_base_class_hook(self, fullname: str
                            ) -> Optional[Callable[[ClassDefContext], None]]:
        return self._find_hook(lambda plugin: plugin.get_base_class_hook(fullname))

    def _find_hook(self, lookup: Callable[[Plugin], T]) -> Optional[T]:
        for plugin in self._plugins:
            hook = lookup(plugin)
            if hook:
                return hook
        return None


class DefaultPlugin(Plugin):
    """Type checker plugin that is enabled by default."""

    def get_function_hook(self, fullname: str
                          ) -> Optional[Callable[[FunctionContext], Type]]:
        if fullname == 'contextlib.contextmanager':
            return contextmanager_callback
        elif fullname == 'builtins.open' and self.python_version[0] == 3:
            return open_callback
        return None

    def get_method_signature_hook(self, fullname: str
                                  ) -> Optional[Callable[[MethodSigContext], CallableType]]:
        if fullname == 'typing.Mapping.get':
            return typed_dict_get_signature_callback
        return None

    def get_method_hook(self, fullname: str
                        ) -> Optional[Callable[[MethodContext], Type]]:
        if fullname == 'typing.Mapping.get':
            return typed_dict_get_callback
        elif fullname == 'builtins.int.__pow__':
            return int_pow_callback
        return None

    def get_class_decorator_hook(self, fullname: str
                                 ) -> Optional[Callable[[ClassDefContext], None]]:
        from mypy.plugins import attrs
        from mypy.plugins import dataclasses

        if fullname in attrs.attr_class_makers:
            return attrs.attr_class_maker_callback
        elif fullname in attrs.attr_dataclass_makers:
            return partial(
                attrs.attr_class_maker_callback,
                auto_attribs_default=True
            )
        elif fullname in dataclasses.dataclass_makers:
            return dataclasses.dataclass_class_maker_callback
        return None


def open_callback(ctx: FunctionContext) -> Type:
    """Infer a better return type for 'open'.

    Infer TextIO or BinaryIO as the return value if the mode argument is not
    given or is a literal.
    """
    mode = None
    if not ctx.arg_types or len(ctx.arg_types[1]) != 1:
        mode = 'r'
    elif isinstance(ctx.args[1][0], StrExpr):
        mode = ctx.args[1][0].value
    if mode is not None:
        assert isinstance(ctx.default_return_type, Instance)
        if 'b' in mode:
            return ctx.api.named_generic_type('typing.BinaryIO', [])
        else:
            return ctx.api.named_generic_type('typing.TextIO', [])
    return ctx.default_return_type


def contextmanager_callback(ctx: FunctionContext) -> Type:
    """Infer a better return type for 'contextlib.contextmanager'."""
    # Be defensive, just in case.
    if ctx.arg_types and len(ctx.arg_types[0]) == 1:
        arg_type = ctx.arg_types[0][0]
        if (isinstance(arg_type, CallableType)
                and isinstance(ctx.default_return_type, CallableType)):
            # The stub signature doesn't preserve information about arguments so
            # add them back here.
            return ctx.default_return_type.copy_modified(
                arg_types=arg_type.arg_types,
                arg_kinds=arg_type.arg_kinds,
                arg_names=arg_type.arg_names,
                variables=arg_type.variables,
                is_ellipsis_args=arg_type.is_ellipsis_args)
    return ctx.default_return_type


def typed_dict_get_signature_callback(ctx: MethodSigContext) -> CallableType:
    """Try to infer a better signature type for TypedDict.get.

    This is used to get better type context for the second argument that
    depends on a TypedDict value type.
    """
    signature = ctx.default_signature
    if (isinstance(ctx.type, TypedDictType)
            and len(ctx.args) == 2
            and len(ctx.args[0]) == 1
            and isinstance(ctx.args[0][0], StrExpr)
            and len(signature.arg_types) == 2
            and len(signature.variables) == 1
            and len(ctx.args[1]) == 1):
        key = ctx.args[0][0].value
        value_type = ctx.type.items.get(key)
        ret_type = signature.ret_type
        if value_type:
            default_arg = ctx.args[1][0]
            if (isinstance(value_type, TypedDictType)
                    and isinstance(default_arg, DictExpr)
                    and len(default_arg.items) == 0):
                # Caller has empty dict {} as default for typed dict.
                value_type = value_type.copy_modified(required_keys=set())
            # Tweak the signature to include the value type as context. It's
            # only needed for type inference since there's a union with a type
            # variable that accepts everything.
            tv = TypeVarType(signature.variables[0])
            return signature.copy_modified(
                arg_types=[signature.arg_types[0],
                           UnionType.make_simplified_union([value_type, tv])],
                ret_type=ret_type)
    return signature


def typed_dict_get_callback(ctx: MethodContext) -> Type:
    """Infer a precise return type for TypedDict.get with literal first argument."""
    if (isinstance(ctx.type, TypedDictType)
            and len(ctx.arg_types) >= 1
            and len(ctx.arg_types[0]) == 1):
        if isinstance(ctx.args[0][0], StrExpr):
            key = ctx.args[0][0].value
            value_type = ctx.type.items.get(key)
            if value_type:
                if len(ctx.arg_types) == 1:
                    return UnionType.make_simplified_union([value_type, NoneTyp()])
                elif (len(ctx.arg_types) == 2 and len(ctx.arg_types[1]) == 1
                      and len(ctx.args[1]) == 1):
                    default_arg = ctx.args[1][0]
                    if (isinstance(default_arg, DictExpr) and len(default_arg.items) == 0
                            and isinstance(value_type, TypedDictType)):
                        # Special case '{}' as the default for a typed dict type.
                        return value_type.copy_modified(required_keys=set())
                    else:
                        return UnionType.make_simplified_union([value_type, ctx.arg_types[1][0]])
            else:
                ctx.api.msg.typeddict_key_not_found(ctx.type, key, ctx.context)
                return AnyType(TypeOfAny.from_error)
    return ctx.default_return_type


def int_pow_callback(ctx: MethodContext) -> Type:
    """Infer a more precise return type for int.__pow__."""
    if (len(ctx.arg_types) == 1
            and len(ctx.arg_types[0]) == 1):
        arg = ctx.args[0][0]
        if isinstance(arg, IntExpr):
            exponent = arg.value
        elif isinstance(arg, UnaryExpr) and arg.op == '-' and isinstance(arg.expr, IntExpr):
            exponent = -arg.expr.value
        else:
            # Right operand not an int literal or a negated literal -- give up.
            return ctx.default_return_type
        if exponent >= 0:
            return ctx.api.named_generic_type('builtins.int', [])
        else:
            return ctx.api.named_generic_type('builtins.float', [])
    return ctx.default_return_type



#!/usr/bin/env python3
"""Script to run mypy's test suite against this version of typeshed."""

from pathlib import Path
import shutil
import subprocess
import sys
import tempfile


if __name__ == '__main__':
    with tempfile.TemporaryDirectory() as tempdir:
        dirpath = Path(tempdir)
        subprocess.run(['python2.7', '-m', 'pip', 'install', '--user', 'typing'], check=True)
        subprocess.run(['git', 'clone', '--depth', '1', 'git://github.com/python/mypy',
                        str(dirpath / 'mypy')], check=True)
        subprocess.run([sys.executable, '-m', 'pip', 'install', '-U', '-r',
                        str(dirpath / 'mypy/test-requirements.txt')], check=True)
        shutil.copytree('stdlib', str(dirpath / 'mypy/typeshed/stdlib'))
        shutil.copytree('third_party', str(dirpath / 'mypy/typeshed/third_party'))
        try:
            subprocess.run(['./runtests.py', '-j12'], cwd=str(dirpath / 'mypy'), check=True)
        except subprocess.CalledProcessError as e:
            print('mypy tests failed', file=sys.stderr)
            sys.exit(e.returncode)
        else:
            print('mypy tests succeeded', file=sys.stderr)
            sys.exit(0)



"""Test cases for the constraint solver used in type inference."""

from typing import List, Union, Tuple, Optional

from mypy.test.helpers import Suite, assert_equal
from mypy.constraints import SUPERTYPE_OF, SUBTYPE_OF, Constraint
from mypy.solve import solve_constraints
from mypy.test.typefixture import TypeFixture
from mypy.types import Type, TypeVarType, TypeVarId


class SolveSuite(Suite):
    def setUp(self) -> None:
        self.fx = TypeFixture()

    def test_empty_input(self) -> None:
        self.assert_solve([], [], [])

    def test_simple_supertype_constraints(self) -> None:
        self.assert_solve([self.fx.t.id],
                          [self.supc(self.fx.t, self.fx.a)],
                          [(self.fx.a, self.fx.o)])
        self.assert_solve([self.fx.t.id],
                          [self.supc(self.fx.t, self.fx.a),
                           self.supc(self.fx.t, self.fx.b)],
                          [(self.fx.a, self.fx.o)])

    def test_simple_subtype_constraints(self) -> None:
        self.assert_solve([self.fx.t.id],
                          [self.subc(self.fx.t, self.fx.a)],
                          [self.fx.a])
        self.assert_solve([self.fx.t.id],
                          [self.subc(self.fx.t, self.fx.a),
                           self.subc(self.fx.t, self.fx.b)],
                          [self.fx.b])

    def test_both_kinds_of_constraints(self) -> None:
        self.assert_solve([self.fx.t.id],
                          [self.supc(self.fx.t, self.fx.b),
                           self.subc(self.fx.t, self.fx.a)],
                          [(self.fx.b, self.fx.a)])

    def test_unsatisfiable_constraints(self) -> None:
        # The constraints are impossible to satisfy.
        self.assert_solve([self.fx.t.id],
                          [self.supc(self.fx.t, self.fx.a),
                           self.subc(self.fx.t, self.fx.b)],
                          [None])

    def test_exactly_specified_result(self) -> None:
        self.assert_solve([self.fx.t.id],
                          [self.supc(self.fx.t, self.fx.b),
                           self.subc(self.fx.t, self.fx.b)],
                          [(self.fx.b, self.fx.b)])

    def test_multiple_variables(self) -> None:
        self.assert_solve([self.fx.t.id, self.fx.s.id],
                          [self.supc(self.fx.t, self.fx.b),
                           self.supc(self.fx.s, self.fx.c),
                           self.subc(self.fx.t, self.fx.a)],
                          [(self.fx.b, self.fx.a), (self.fx.c, self.fx.o)])

    def test_no_constraints_for_var(self) -> None:
        self.assert_solve([self.fx.t.id],
                          [],
                          [self.fx.uninhabited])
        self.assert_solve([self.fx.t.id, self.fx.s.id],
                          [],
                          [self.fx.uninhabited, self.fx.uninhabited])
        self.assert_solve([self.fx.t.id, self.fx.s.id],
                          [self.supc(self.fx.s, self.fx.a)],
                          [self.fx.uninhabited, (self.fx.a, self.fx.o)])

    def test_simple_constraints_with_dynamic_type(self) -> None:
        self.assert_solve([self.fx.t.id],
                          [self.supc(self.fx.t, self.fx.anyt)],
                          [(self.fx.anyt, self.fx.anyt)])
        self.assert_solve([self.fx.t.id],
                          [self.supc(self.fx.t, self.fx.anyt),
                           self.supc(self.fx.t, self.fx.anyt)],
                          [(self.fx.anyt, self.fx.anyt)])
        self.assert_solve([self.fx.t.id],
                          [self.supc(self.fx.t, self.fx.anyt),
                           self.supc(self.fx.t, self.fx.a)],
                          [(self.fx.anyt, self.fx.anyt)])

        self.assert_solve([self.fx.t.id],
                          [self.subc(self.fx.t, self.fx.anyt)],
                          [(self.fx.anyt, self.fx.anyt)])
        self.assert_solve([self.fx.t.id],
                          [self.subc(self.fx.t, self.fx.anyt),
                           self.subc(self.fx.t, self.fx.anyt)],
                          [(self.fx.anyt, self.fx.anyt)])
        # self.assert_solve([self.fx.t.id],
        #                   [self.subc(self.fx.t, self.fx.anyt),
        #                    self.subc(self.fx.t, self.fx.a)],
        #                   [(self.fx.anyt, self.fx.anyt)])
        # TODO: figure out what this should be after changes to meet(any, X)

    def test_both_normal_and_any_types_in_results(self) -> None:
        # If one of the bounds is any, we promote the other bound to
        # any as well, since otherwise the type range does not make sense.
        self.assert_solve([self.fx.t.id],
                          [self.supc(self.fx.t, self.fx.a),
                           self.subc(self.fx.t, self.fx.anyt)],
                          [(self.fx.anyt, self.fx.anyt)])

        self.assert_solve([self.fx.t.id],
                          [self.supc(self.fx.t, self.fx.anyt),
                           self.subc(self.fx.t, self.fx.a)],
                          [(self.fx.anyt, self.fx.anyt)])

    def assert_solve(self,
                     vars: List[TypeVarId],
                     constraints: List[Constraint],
                     results: List[Union[None, Type, Tuple[Type, Type]]],
                     ) -> None:
        res = []  # type: List[Optional[Type]]
        for r in results:
            if isinstance(r, tuple):
                res.append(r[0])
            else:
                res.append(r)
        actual = solve_constraints(vars, constraints)
        assert_equal(str(actual), str(res))

    def supc(self, type_var: TypeVarType, bound: Type) -> Constraint:
        return Constraint(type_var.id, SUPERTYPE_OF, bound)

    def subc(self, type_var: TypeVarType, bound: Type) -> Constraint:
        return Constraint(type_var.id, SUBTYPE_OF, bound)



from typing import Any
from werkzeug.wrappers import BaseRequest as Request, BaseResponse as Response

PIN_TIME = ...  # type: Any

def hash_pin(pin): ...
def get_machine_id(): ...

class _ConsoleFrame:
    console = ...  # type: Any
    id = ...  # type: Any
    def __init__(self, namespace): ...

def get_pin_and_cookie_name(app): ...

class DebuggedApplication:
    app = ...  # type: Any
    evalex = ...  # type: Any
    frames = ...  # type: Any
    tracebacks = ...  # type: Any
    request_key = ...  # type: Any
    console_path = ...  # type: Any
    console_init_func = ...  # type: Any
    show_hidden_frames = ...  # type: Any
    secret = ...  # type: Any
    pin_logging = ...  # type: Any
    pin = ...  # type: Any
    def __init__(self, app, evalex=False, request_key='', console_path='', console_init_func=None, show_hidden_frames=False, lodgeit_url=None, pin_security=True, pin_logging=True): ...
    @property
    def pin_cookie_name(self): ...
    def debug_application(self, environ, start_response): ...
    def execute_command(self, request, command, frame): ...
    def display_console(self, request): ...
    def paste_traceback(self, request, traceback): ...
    def get_resource(self, request, filename): ...
    def check_pin_trust(self, environ): ...
    def pin_auth(self, request): ...
    def log_pin_request(self): ...
    def __call__(self, environ, start_response): ...



# Stubs for smtpd (Python 2 and 3)
import sys
import socket
import asyncore
import asynchat

from typing import Any, DefaultDict, List, Optional, Text, Tuple, Type

_Address = Tuple[str, int]  # (host, port)


class SMTPChannel(asynchat.async_chat):
    COMMAND: int
    DATA: int

    if sys.version_info >= (3, 3):
        command_size_limits: DefaultDict[str, int]

    if sys.version_info >= (3,):
        smtp_server: SMTPServer
        conn: socket.socket
        addr: Any
        received_lines: List[Text]
        smtp_state: int
        seen_greeting: str
        mailfrom: str
        rcpttos: List[str]
        received_data: str
        fqdn: str
        peer: str

        command_size_limit: int
        data_size_limit: int

    if sys.version_info >= (3, 5):
        enable_SMTPUTF8: bool

    if sys.version_info >= (3, 3):
        @property
        def max_command_size_limit(self) -> int: ...

    if sys.version_info >= (3, 5):
        def __init__(self, server: SMTPServer, conn: socket.socket, addr: Any, data_size_limit: int = ...,
                     map: Optional[asyncore._maptype] = ..., enable_SMTPUTF8: bool = ..., decode_data: bool = ...) -> None: ...
    elif sys.version_info >= (3, 4):
        def __init__(self, server: SMTPServer, conn: socket.socket, addr: Any, data_size_limit: int = ...,
                     map: Optional[asyncore._maptype] = ...) -> None: ...
    else:
        def __init__(self, server: SMTPServer, conn: socket.socket, addr: Any, data_size_limit: int = ...) -> None: ...
    def push(self, msg: bytes) -> None: ...
    def collect_incoming_data(self, data: bytes) -> None: ...
    def found_terminator(self) -> None: ...
    def smtp_HELO(self, arg: str) -> None: ...
    def smtp_NOOP(self, arg: str) -> None: ...
    def smtp_QUIT(self, arg: str) -> None: ...
    def smtp_MAIL(self, arg: str) -> None: ...
    def smtp_RCPT(self, arg: str) -> None: ...
    def smtp_RSET(self, arg: str) -> None: ...
    def smtp_DATA(self, arg: str) -> None: ...
    if sys.version_info >= (3, 3):
        def smtp_EHLO(self, arg: str) -> None: ...
        def smtp_HELP(self, arg: str) -> None: ...
        def smtp_VRFY(self, arg: str) -> None: ...
        def smtp_EXPN(self, arg: str) -> None: ...

class SMTPServer(asyncore.dispatcher):
    channel_class: Type[SMTPChannel]

    data_size_limit: int
    enable_SMTPUTF8: bool

    if sys.version_info >= (3, 5):
        def __init__(self, localaddr: _Address, remoteaddr: _Address,
                     data_size_limit: int = ..., map: Optional[asyncore._maptype] = ...,
                     enable_SMTPUTF8: bool = ..., decode_data: bool = ...) -> None: ...
    elif sys.version_info >= (3, 4):
        def __init__(self, localaddr: _Address, remoteaddr: _Address,
                     data_size_limit: int = ..., map: Optional[asyncore._maptype] = ...) -> None: ...
    else:
        def __init__(self, localaddr: _Address, remoteaddr: _Address,
                     data_size_limit: int = ...) -> None: ...
    def handle_accepted(self, conn: socket.socket, addr: Any) -> None: ...
    def process_message(self, peer: _Address, mailfrom: str, rcpttos: List[Text], data: str, **kwargs: Any) -> Optional[str]: ...

class DebuggingServer(SMTPServer): ...
class PureProxy(SMTPServer): ...
class MailmanProxy(PureProxy): ...



from typing import Any

class KeyFile:
    key = ...  # type: Any
    location = ...  # type: int
    closed = ...  # type: bool
    softspace = ...  # type: int
    mode = ...  # type: str
    encoding = ...  # type: str
    errors = ...  # type: str
    newlines = ...  # type: str
    name = ...  # type: Any
    def __init__(self, key) -> None: ...
    def tell(self): ...
    def seek(self, pos, whence: Any = ...): ...
    def read(self, size): ...
    def close(self): ...
    def isatty(self): ...
    def getkey(self): ...
    def write(self, buf): ...
    def fileno(self): ...
    def flush(self): ...
    def next(self): ...
    def readinto(self): ...
    def readline(self): ...
    def readlines(self): ...
    def truncate(self): ...
    def writelines(self): ...
    def xreadlines(self): ...



"""Generic abstract syntax tree node visitor"""

from abc import abstractmethod
from typing import TypeVar, Generic

if False:
    # break import cycle only needed for mypy
    import mypy.nodes


T = TypeVar('T')


class ExpressionVisitor(Generic[T]):
    @abstractmethod
    def visit_int_expr(self, o: 'mypy.nodes.IntExpr') -> T:
        pass

    @abstractmethod
    def visit_str_expr(self, o: 'mypy.nodes.StrExpr') -> T:
        pass

    @abstractmethod
    def visit_bytes_expr(self, o: 'mypy.nodes.BytesExpr') -> T:
        pass

    @abstractmethod
    def visit_unicode_expr(self, o: 'mypy.nodes.UnicodeExpr') -> T:
        pass

    @abstractmethod
    def visit_float_expr(self, o: 'mypy.nodes.FloatExpr') -> T:
        pass

    @abstractmethod
    def visit_complex_expr(self, o: 'mypy.nodes.ComplexExpr') -> T:
        pass

    @abstractmethod
    def visit_ellipsis(self, o: 'mypy.nodes.EllipsisExpr') -> T:
        pass

    @abstractmethod
    def visit_star_expr(self, o: 'mypy.nodes.StarExpr') -> T:
        pass

    @abstractmethod
    def visit_name_expr(self, o: 'mypy.nodes.NameExpr') -> T:
        pass

    @abstractmethod
    def visit_member_expr(self, o: 'mypy.nodes.MemberExpr') -> T:
        pass

    @abstractmethod
    def visit_yield_from_expr(self, o: 'mypy.nodes.YieldFromExpr') -> T:
        pass

    @abstractmethod
    def visit_yield_expr(self, o: 'mypy.nodes.YieldExpr') -> T:
        pass

    @abstractmethod
    def visit_call_expr(self, o: 'mypy.nodes.CallExpr') -> T:
        pass

    @abstractmethod
    def visit_op_expr(self, o: 'mypy.nodes.OpExpr') -> T:
        pass

    @abstractmethod
    def visit_comparison_expr(self, o: 'mypy.nodes.ComparisonExpr') -> T:
        pass

    @abstractmethod
    def visit_cast_expr(self, o: 'mypy.nodes.CastExpr') -> T:
        pass

    @abstractmethod
    def visit_reveal_expr(self, o: 'mypy.nodes.RevealExpr') -> T:
        pass

    @abstractmethod
    def visit_super_expr(self, o: 'mypy.nodes.SuperExpr') -> T:
        pass

    @abstractmethod
    def visit_unary_expr(self, o: 'mypy.nodes.UnaryExpr') -> T:
        pass

    @abstractmethod
    def visit_list_expr(self, o: 'mypy.nodes.ListExpr') -> T:
        pass

    @abstractmethod
    def visit_dict_expr(self, o: 'mypy.nodes.DictExpr') -> T:
        pass

    @abstractmethod
    def visit_tuple_expr(self, o: 'mypy.nodes.TupleExpr') -> T:
        pass

    @abstractmethod
    def visit_set_expr(self, o: 'mypy.nodes.SetExpr') -> T:
        pass

    @abstractmethod
    def visit_index_expr(self, o: 'mypy.nodes.IndexExpr') -> T:
        pass

    @abstractmethod
    def visit_type_application(self, o: 'mypy.nodes.TypeApplication') -> T:
        pass

    @abstractmethod
    def visit_lambda_expr(self, o: 'mypy.nodes.LambdaExpr') -> T:
        pass

    @abstractmethod
    def visit_list_comprehension(self, o: 'mypy.nodes.ListComprehension') -> T:
        pass

    @abstractmethod
    def visit_set_comprehension(self, o: 'mypy.nodes.SetComprehension') -> T:
        pass

    @abstractmethod
    def visit_dictionary_comprehension(self, o: 'mypy.nodes.DictionaryComprehension') -> T:
        pass

    @abstractmethod
    def visit_generator_expr(self, o: 'mypy.nodes.GeneratorExpr') -> T:
        pass

    @abstractmethod
    def visit_slice_expr(self, o: 'mypy.nodes.SliceExpr') -> T:
        pass

    @abstractmethod
    def visit_conditional_expr(self, o: 'mypy.nodes.ConditionalExpr') -> T:
        pass

    @abstractmethod
    def visit_backquote_expr(self, o: 'mypy.nodes.BackquoteExpr') -> T:
        pass

    @abstractmethod
    def visit_type_var_expr(self, o: 'mypy.nodes.TypeVarExpr') -> T:
        pass

    @abstractmethod
    def visit_type_alias_expr(self, o: 'mypy.nodes.TypeAliasExpr') -> T:
        pass

    @abstractmethod
    def visit_namedtuple_expr(self, o: 'mypy.nodes.NamedTupleExpr') -> T:
        pass

    @abstractmethod
    def visit_enum_call_expr(self, o: 'mypy.nodes.EnumCallExpr') -> T:
        pass

    @abstractmethod
    def visit_typeddict_expr(self, o: 'mypy.nodes.TypedDictExpr') -> T:
        pass

    @abstractmethod
    def visit_newtype_expr(self, o: 'mypy.nodes.NewTypeExpr') -> T:
        pass

    @abstractmethod
    def visit__promote_expr(self, o: 'mypy.nodes.PromoteExpr') -> T:
        pass

    @abstractmethod
    def visit_await_expr(self, o: 'mypy.nodes.AwaitExpr') -> T:
        pass

    @abstractmethod
    def visit_temp_node(self, o: 'mypy.nodes.TempNode') -> T:
        pass


class StatementVisitor(Generic[T]):
    # Definitions

    @abstractmethod
    def visit_assignment_stmt(self, o: 'mypy.nodes.AssignmentStmt') -> T:
        pass

    @abstractmethod
    def visit_for_stmt(self, o: 'mypy.nodes.ForStmt') -> T:
        pass

    @abstractmethod
    def visit_with_stmt(self, o: 'mypy.nodes.WithStmt') -> T:
        pass

    @abstractmethod
    def visit_del_stmt(self, o: 'mypy.nodes.DelStmt') -> T:
        pass

    @abstractmethod
    def visit_func_def(self, o: 'mypy.nodes.FuncDef') -> T:
        pass

    @abstractmethod
    def visit_overloaded_func_def(self, o: 'mypy.nodes.OverloadedFuncDef') -> T:
        pass

    @abstractmethod
    def visit_class_def(self, o: 'mypy.nodes.ClassDef') -> T:
        pass

    @abstractmethod
    def visit_global_decl(self, o: 'mypy.nodes.GlobalDecl') -> T:
        pass

    @abstractmethod
    def visit_nonlocal_decl(self, o: 'mypy.nodes.NonlocalDecl') -> T:
        pass

    @abstractmethod
    def visit_decorator(self, o: 'mypy.nodes.Decorator') -> T:
        pass

    @abstractmethod
    def visit_var(self, o: 'mypy.nodes.Var') -> T:
        pass

    # Module structure

    @abstractmethod
    def visit_import(self, o: 'mypy.nodes.Import') -> T:
        pass

    @abstractmethod
    def visit_import_from(self, o: 'mypy.nodes.ImportFrom') -> T:
        pass

    @abstractmethod
    def visit_import_all(self, o: 'mypy.nodes.ImportAll') -> T:
        pass

    # Statements

    @abstractmethod
    def visit_block(self, o: 'mypy.nodes.Block') -> T:
        pass

    @abstractmethod
    def visit_expression_stmt(self, o: 'mypy.nodes.ExpressionStmt') -> T:
        pass

    @abstractmethod
    def visit_operator_assignment_stmt(self, o: 'mypy.nodes.OperatorAssignmentStmt') -> T:
        pass

    @abstractmethod
    def visit_while_stmt(self, o: 'mypy.nodes.WhileStmt') -> T:
        pass

    @abstractmethod
    def visit_return_stmt(self, o: 'mypy.nodes.ReturnStmt') -> T:
        pass

    @abstractmethod
    def visit_assert_stmt(self, o: 'mypy.nodes.AssertStmt') -> T:
        pass

    @abstractmethod
    def visit_if_stmt(self, o: 'mypy.nodes.IfStmt') -> T:
        pass

    @abstractmethod
    def visit_break_stmt(self, o: 'mypy.nodes.BreakStmt') -> T:
        pass

    @abstractmethod
    def visit_continue_stmt(self, o: 'mypy.nodes.ContinueStmt') -> T:
        pass

    @abstractmethod
    def visit_pass_stmt(self, o: 'mypy.nodes.PassStmt') -> T:
        pass

    @abstractmethod
    def visit_raise_stmt(self, o: 'mypy.nodes.RaiseStmt') -> T:
        pass

    @abstractmethod
    def visit_try_stmt(self, o: 'mypy.nodes.TryStmt') -> T:
        pass

    @abstractmethod
    def visit_print_stmt(self, o: 'mypy.nodes.PrintStmt') -> T:
        pass

    @abstractmethod
    def visit_exec_stmt(self, o: 'mypy.nodes.ExecStmt') -> T:
        pass


class NodeVisitor(Generic[T], ExpressionVisitor[T], StatementVisitor[T]):
    """Empty base class for parse tree node visitors.

    The T type argument specifies the return type of the visit
    methods. As all methods defined here return None by default,
    subclasses do not always need to override all the methods.

    TODO make the default return value explicit
    """

    # Not in superclasses:

    def visit_mypy_file(self, o: 'mypy.nodes.MypyFile') -> T:
        pass

    # Module structure

    def visit_import(self, o: 'mypy.nodes.Import') -> T:
        pass

    def visit_import_from(self, o: 'mypy.nodes.ImportFrom') -> T:
        pass

    def visit_import_all(self, o: 'mypy.nodes.ImportAll') -> T:
        pass

    # Definitions

    def visit_func_def(self, o: 'mypy.nodes.FuncDef') -> T:
        pass

    def visit_overloaded_func_def(self,
                                  o: 'mypy.nodes.OverloadedFuncDef') -> T:
        pass

    def visit_class_def(self, o: 'mypy.nodes.ClassDef') -> T:
        pass

    def visit_global_decl(self, o: 'mypy.nodes.GlobalDecl') -> T:
        pass

    def visit_nonlocal_decl(self, o: 'mypy.nodes.NonlocalDecl') -> T:
        pass

    def visit_decorator(self, o: 'mypy.nodes.Decorator') -> T:
        pass

    def visit_var(self, o: 'mypy.nodes.Var') -> T:
        pass

    def visit_type_alias(self, o: 'mypy.nodes.TypeAlias') -> T:
        pass

    # Statements

    def visit_block(self, o: 'mypy.nodes.Block') -> T:
        pass

    def visit_expression_stmt(self, o: 'mypy.nodes.ExpressionStmt') -> T:
        pass

    def visit_assignment_stmt(self, o: 'mypy.nodes.AssignmentStmt') -> T:
        pass

    def visit_operator_assignment_stmt(self,
                                       o: 'mypy.nodes.OperatorAssignmentStmt') -> T:
        pass

    def visit_while_stmt(self, o: 'mypy.nodes.WhileStmt') -> T:
        pass

    def visit_for_stmt(self, o: 'mypy.nodes.ForStmt') -> T:
        pass

    def visit_return_stmt(self, o: 'mypy.nodes.ReturnStmt') -> T:
        pass

    def visit_assert_stmt(self, o: 'mypy.nodes.AssertStmt') -> T:
        pass

    def visit_del_stmt(self, o: 'mypy.nodes.DelStmt') -> T:
        pass

    def visit_if_stmt(self, o: 'mypy.nodes.IfStmt') -> T:
        pass

    def visit_break_stmt(self, o: 'mypy.nodes.BreakStmt') -> T:
        pass

    def visit_continue_stmt(self, o: 'mypy.nodes.ContinueStmt') -> T:
        pass

    def visit_pass_stmt(self, o: 'mypy.nodes.PassStmt') -> T:
        pass

    def visit_raise_stmt(self, o: 'mypy.nodes.RaiseStmt') -> T:
        pass

    def visit_try_stmt(self, o: 'mypy.nodes.TryStmt') -> T:
        pass

    def visit_with_stmt(self, o: 'mypy.nodes.WithStmt') -> T:
        pass

    def visit_print_stmt(self, o: 'mypy.nodes.PrintStmt') -> T:
        pass

    def visit_exec_stmt(self, o: 'mypy.nodes.ExecStmt') -> T:
        pass

    # Expressions (default no-op implementation)

    def visit_int_expr(self, o: 'mypy.nodes.IntExpr') -> T:
        pass

    def visit_str_expr(self, o: 'mypy.nodes.StrExpr') -> T:
        pass

    def visit_bytes_expr(self, o: 'mypy.nodes.BytesExpr') -> T:
        pass

    def visit_unicode_expr(self, o: 'mypy.nodes.UnicodeExpr') -> T:
        pass

    def visit_float_expr(self, o: 'mypy.nodes.FloatExpr') -> T:
        pass

    def visit_complex_expr(self, o: 'mypy.nodes.ComplexExpr') -> T:
        pass

    def visit_ellipsis(self, o: 'mypy.nodes.EllipsisExpr') -> T:
        pass

    def visit_star_expr(self, o: 'mypy.nodes.StarExpr') -> T:
        pass

    def visit_name_expr(self, o: 'mypy.nodes.NameExpr') -> T:
        pass

    def visit_member_expr(self, o: 'mypy.nodes.MemberExpr') -> T:
        pass

    def visit_yield_from_expr(self, o: 'mypy.nodes.YieldFromExpr') -> T:
        pass

    def visit_yield_expr(self, o: 'mypy.nodes.YieldExpr') -> T:
        pass

    def visit_call_expr(self, o: 'mypy.nodes.CallExpr') -> T:
        pass

    def visit_op_expr(self, o: 'mypy.nodes.OpExpr') -> T:
        pass

    def visit_comparison_expr(self, o: 'mypy.nodes.ComparisonExpr') -> T:
        pass

    def visit_cast_expr(self, o: 'mypy.nodes.CastExpr') -> T:
        pass

    def visit_reveal_expr(self, o: 'mypy.nodes.RevealExpr') -> T:
        pass

    def visit_super_expr(self, o: 'mypy.nodes.SuperExpr') -> T:
        pass

    def visit_unary_expr(self, o: 'mypy.nodes.UnaryExpr') -> T:
        pass

    def visit_list_expr(self, o: 'mypy.nodes.ListExpr') -> T:
        pass

    def visit_dict_expr(self, o: 'mypy.nodes.DictExpr') -> T:
        pass

    def visit_tuple_expr(self, o: 'mypy.nodes.TupleExpr') -> T:
        pass

    def visit_set_expr(self, o: 'mypy.nodes.SetExpr') -> T:
        pass

    def visit_index_expr(self, o: 'mypy.nodes.IndexExpr') -> T:
        pass

    def visit_type_application(self, o: 'mypy.nodes.TypeApplication') -> T:
        pass

    def visit_lambda_expr(self, o: 'mypy.nodes.LambdaExpr') -> T:
        pass

    def visit_list_comprehension(self, o: 'mypy.nodes.ListComprehension') -> T:
        pass

    def visit_set_comprehension(self, o: 'mypy.nodes.SetComprehension') -> T:
        pass

    def visit_dictionary_comprehension(self, o: 'mypy.nodes.DictionaryComprehension') -> T:
        pass

    def visit_generator_expr(self, o: 'mypy.nodes.GeneratorExpr') -> T:
        pass

    def visit_slice_expr(self, o: 'mypy.nodes.SliceExpr') -> T:
        pass

    def visit_conditional_expr(self, o: 'mypy.nodes.ConditionalExpr') -> T:
        pass

    def visit_backquote_expr(self, o: 'mypy.nodes.BackquoteExpr') -> T:
        pass

    def visit_type_var_expr(self, o: 'mypy.nodes.TypeVarExpr') -> T:
        pass

    def visit_type_alias_expr(self, o: 'mypy.nodes.TypeAliasExpr') -> T:
        pass

    def visit_namedtuple_expr(self, o: 'mypy.nodes.NamedTupleExpr') -> T:
        pass

    def visit_enum_call_expr(self, o: 'mypy.nodes.EnumCallExpr') -> T:
        pass

    def visit_typeddict_expr(self, o: 'mypy.nodes.TypedDictExpr') -> T:
        pass

    def visit_newtype_expr(self, o: 'mypy.nodes.NewTypeExpr') -> T:
        pass

    def visit__promote_expr(self, o: 'mypy.nodes.PromoteExpr') -> T:
        pass

    def visit_await_expr(self, o: 'mypy.nodes.AwaitExpr') -> T:
        pass

    def visit_temp_node(self, o: 'mypy.nodes.TempNode') -> T:
        pass
